{ Version 010518. Copyright  Alexey A.Chernobaev, 1996-2001 }

unit ExtStrm;
{
   OLE IStream  TVStream.
  Implements OLE IStream on TVStream.
}

interface

{$I VCheck.inc}

uses
  Windows, ActiveX, VStream;

type
  TVStreamAdapter = class(TInterfacedObject, IStream)
  protected
    FStream: TVStream;
    FOwnership: Boolean;
  public
    constructor Create(Stream: TVStream; TakeOwnership: Boolean);
    destructor Destroy; override;
    function Read(pv: Pointer; cb: Longint;
      pcbRead: PLongint): HResult; virtual; stdcall;
    function Write(pv: Pointer; cb: Longint;
      pcbWritten: PLongint): HResult; virtual; stdcall;
    function Seek(dlibMove: Largeint; dwOrigin: Longint;
      out libNewPosition: Largeint): HResult; virtual; stdcall;
    function SetSize(libNewSize: Largeint): HResult; virtual; stdcall;
    function CopyTo(stm: IStream; cb: Largeint; out cbRead: Largeint;
      out cbWritten: Largeint): HResult; virtual; stdcall;
    function Commit(grfCommitFlags: Longint): HResult; virtual; stdcall;
    function Revert: HResult; virtual; stdcall;
    function LockRegion(libOffset: Largeint; cb: Largeint;
      dwLockType: Longint): HResult; virtual; stdcall;
    function UnlockRegion(libOffset: Largeint; cb: Largeint;
      dwLockType: Longint): HResult; virtual; stdcall;
    function Stat(out statstg: TStatStg;
      grfStatFlag: Longint): HResult; virtual; stdcall;
    function Clone(out stm: IStream): HResult; virtual; stdcall;
    property Stream: TVStream read FStream;
    property StreamOwnership: Boolean read FOwnership write FOwnership;
  end;

implementation

constructor TVStreamAdapter.Create(Stream: TVStream; TakeOwnership: Boolean);
begin
  inherited Create;
  FStream:=Stream;
  FOwnership:=TakeOwnership;
end;

destructor TVStreamAdapter.Destroy;
begin
  if FOwnership then FStream.Free;
  inherited Destroy;
end;

function TVStreamAdapter.Read(pv: Pointer; cb: Longint; pcbRead: PLongint): HResult;
var
  NumRead: Longint;
begin
  try
    if pv = nil then begin
      Result:=STG_E_INVALIDPOINTER;
      Exit;
    end;
    NumRead:=FStream.ReadFunc(pv^, cb);
    if pcbRead <> nil then pcbRead^:=NumRead;
    Result:=S_OK;
  except
    Result:=S_FALSE;
  end;
end;

function TVStreamAdapter.Write(pv: Pointer; cb: Longint; pcbWritten: PLongint): HResult;
begin
  try
    if pv = nil then begin
      Result:=STG_E_INVALIDPOINTER;
      Exit;
    end;
    FStream.WriteProc(pv^, cb);
    if pcbWritten <> nil then pcbWritten^:=cb;
    Result:=S_OK;
  except
    Result:=STG_E_CANTSAVE;
  end;
end;

function TVStreamAdapter.Seek(dlibMove: Largeint; dwOrigin: Longint;
  out libNewPosition: Largeint): HResult;
begin
  try
    Case dwOrigin of
      STREAM_SEEK_SET: FStream.Seek(LongInt(dlibMove));
      STREAM_SEEK_CUR: FStream.SeekBy(LongInt(dlibMove));
      STREAM_SEEK_END: FStream.Seek(FStream.Size + LongInt(dlibMove));
    Else
      begin
        Result:=STG_E_INVALIDFUNCTION;
        Exit;
      end;
    End;
    if @libNewPosition <> nil then libNewPosition:=FStream.Position;
    Result:=S_OK;
  except
    Result:=STG_E_INVALIDPOINTER;
  end;
end;

function TVStreamAdapter.SetSize(libNewSize: Largeint): HResult;
begin
  try
    FStream.Size:=LongInt(libNewSize);
    if libNewSize <> FStream.Size then
      Result:=E_FAIL
    else
      Result:=S_OK;
  except
    Result:=E_UNEXPECTED;
  end;
end;

function TVStreamAdapter.CopyTo(stm: IStream; cb: Largeint; out cbRead: Largeint;
  out cbWritten: Largeint): HResult;
const
  MaxBufSize = 1024 * 1024; // 1mb
var
  Buffer: Pointer;
  BufSize, N, I: Integer;
  BytesRead, BytesWritten, W: LargeInt;
begin
  Result:=S_OK;
  BytesRead:=0;
  BytesWritten:=0;
  try
    if cb > MaxBufSize then
      BufSize:=MaxBufSize
    else
      BufSize:=Integer(cb);
    GetMem(Buffer, BufSize);
    try
      while cb > 0 do begin
        if cb > MaxInt then
          I:=MaxInt
        else
          I:=cb;
        while I > 0 do begin
          if I > BufSize then N:=BufSize else N:=I;
          Inc(BytesRead, FStream.ReadFunc(Buffer^, N));
          W:=0;
          Result:=stm.Write(Buffer, N, @W);
          Inc(BytesWritten, W);
          if (Result = S_OK) and (Integer(W) <> N) then Result:=E_FAIL;
          if Result <> S_OK then Exit;
          Dec(I, N);
        end;
        Dec(cb, I);
      end;
    finally
      FreeMem(Buffer);
      if @cbWritten <> nil then cbWritten:=BytesWritten;
      if @cbRead <> nil then cbRead:=BytesRead;
    end;
  except
    Result:=E_UNEXPECTED;
  end;
end;

function TVStreamAdapter.Commit(grfCommitFlags: Longint): HResult;
begin
  Result:=S_OK;
end;

function TVStreamAdapter.Revert: HResult;
begin
  Result:=STG_E_REVERTED;
end;

function TVStreamAdapter.LockRegion(libOffset: Largeint; cb: Largeint;
  dwLockType: Longint): HResult;
begin
  Result:=STG_E_INVALIDFUNCTION;
end;

function TVStreamAdapter.UnlockRegion(libOffset: Largeint; cb: Largeint;
  dwLockType: Longint): HResult;
begin
  Result:=STG_E_INVALIDFUNCTION;
end;

function TVStreamAdapter.Stat(out statstg: TStatStg; grfStatFlag: Longint): HResult;
begin
  Result:=S_OK;
  try
    if (@statstg <> nil) then
      With statstg do begin
        dwType:=STGTY_STREAM;
        cbSize:=FStream.Size;
        mTime.dwLowDateTime:=0;
        mTime.dwHighDateTime:=0;
        cTime.dwLowDateTime:=0;
        cTime.dwHighDateTime:=0;
        aTime.dwLowDateTime:=0;
        aTime.dwHighDateTime:=0;
        grfLocksSupported:=LOCK_WRITE;
      end;
  except
    Result:=E_UNEXPECTED;
  end;
end;

function TVStreamAdapter.Clone(out stm: IStream): HResult;
begin
  Result:=E_NOTIMPL;
end;

end.
