{ Version 010703. Copyright  Alexey A.Chernobaev, 1996-2001 }

unit ExtSys;

interface

{$I VCheck.inc}
{$IFDEF V_D6}{$J+}{$ENDIF}

uses
  SysUtils, ExtType;

const
  MemoryReserve: UInt32 =  1048576; { 1Mb }

function IntMin(A, B: Integer): Integer;
{ Result:=min(A, B) }

function IntMax(A, B: Integer): Integer;
{ Result:=max(A, B) }

function FloatMin(A, B: Float): Float;
{ Result:=min(A, B) }

function FloatMax(A, B: Float): Float;
{ Result:=max(A, B) }

function IntMinMax(Value, Floor, Ceil: Integer): Integer;
{ see code }

function PhysicalMemorySize: UInt32;
{     }
{ returns the physical memory size }

function PhysicalMemoryFree: UInt32;
{      }
{ returns the free physical memory size }

function HaveMemoryReserve: Boolean;
{  True,       MemoryReserve  
  ;    ,  ,    
  "" ,      "",
      (, - ) }
{ returns True iff there's not less then MemoryReserve bytes of free physical
  memory in the system; this function can be used to determine is there any
  "spare" memory in the system which can be used for storing some "unnecessary"
  but useful in terms of efficiency data (e.g. some buffers) }

function MemEqual(const X, Y; Count: Integer): Boolean;
{    X  Y  Count     True, 
     Count = 0 }
{ compares the memory blocks X and Y of length Count bytes each and returns True
  iff they are equal or Count = 0 }

procedure MemExchange(var X, Y; Count: Integer);
{     X  Y  Count ;  
     }
{ echanges data in the memory blocks X and Y of length Count bytes; the memory
  blocks must not overlap }

function FindInBuf(const SearchBytes; SearchCount: Integer; const Buffer;
  BufferLength: Integer): Integer;
{      SearchBytes  SearchCount
    Buffer  BufferLength     ,
     ,  -1,    }
{ finds the first occurrence of the pattern SearchBytes of length SearchCount in
  the buffer Buffer of length BufferLength and returns the index of this
  occurrence or -1 if the pattern was not found }

procedure FillPattern(var X; Count: Integer; const Pattern; PatternLength: Integer);
{    X  Count ,  Pattern  PatternLength
     (   FillChar) }
{ fills the memory block X of length Count bytes with the pattern Pattern of
  length PatternLength bytes (the extended variant of the FillChar procedure) }
procedure FillValue16(var X; Value: Int16; Count: Integer);
{    X  (Count * 2) ,  Count  Value;
       Value   Int16 ( 
  Range Check Error) }
{ fills the memory block X of length (Count * 2) bytes with Count copies of
  Value; it's recommended to cast Value to Int16 (to avoid Range Check Error) }
procedure FillValue32(var X; Value: Int32; Count: Integer);
{    X  (Count * 4) ,  Count  Value;
       Value   Int32 ( 
  Range Check Error) }
{ fills the memory block X of length (Count * 4) bytes with Count copies of
  Value; it's recommended to cast Value to Int32 (to avoid Range Check Error) }
procedure FillValue64(var X; const Value; Count: Integer);
{    X  (Count * 8) ,  Count  64-
   Value }
{ fills the memory block X of length (Count * 8) bytes with Count copies of
  64-bit value Value }
procedure FillValue80(var X; const Value; Count: Integer);
{    X  (Count * 10) ,  Count  80-
   Value }
{ fills the memory block X of length (Count * 10) bytes with Count copies of
  80-bit value Value }

function CountEqualToPattern(const X; Count: Integer; const Pattern;
  PatternLength: Integer): Integer;
{    Pattern  PatternLength    X,
    Count   PatternLength  }
{ counts the number of occurrences of the pattern Pattern of length PatternLength
  in the memory block X containing Count elements with length PatternLength each }
function CountEqualToValue8(const X; Value: Int8; Count: Integer): Integer;
function CountEqualToValue16(const X; Value: Int16; Count: Integer): Integer;
function CountEqualToValue32(const X; Value: Int32; Count: Integer): Integer;
function CountEqualToValue64(const X; const Value; Count: Integer): Integer;
function CountEqualToValue80(const X; const Value; Count: Integer): Integer;

function IndexOfPattern(const X; Count: Integer; const Pattern;
  PatternLength: Integer): Integer;
{     Pattern  PatternLength 
    X,   Count   PatternLength  }
{ returns the index of the first occurrence of Pattern of length PatternLength
  in the memory block X containing Count elements of length PatternLength each }
function IndexOfValue8(const X; Value: Int8; Count: Integer): Integer;
function IndexOfValue16(const X; Value: Int16; Count: Integer): Integer;
function IndexOfValue32(const X; Value: Int32; Count: Integer): Integer;
function IndexOfValue64(const X; const Value; Count: Integer): Integer;
function IndexOfValue80(const X; const Value; Count: Integer): Integer;

function LastIndexOfPattern(const X; Count: Integer; const Pattern;
  PatternLength: Integer): Integer;
{     Pattern  PatternLength 
    X,   Count   PatternLength  }
{ returns the index of the last occurrence of Pattern of length PatternLength in
  the memory block X containing Count elements of length PatternLength each }
function LastIndexOfValue8(const X; Value: Int8; Count: Integer): Integer;
function LastIndexOfValue16(const X; Value: Int16; Count: Integer): Integer;
function LastIndexOfValue32(const X; Value: Int32; Count: Integer): Integer;
function LastIndexOfValue64(const X; const Value; Count: Integer): Integer;
function LastIndexOfValue80(const X; const Value; Count: Integer): Integer;

function ReplaceValue8(const X; FromValue, ToValue: Int8; Count: Integer): Integer;
{    FromValue  ToValue    X  Count }
{ replaces all occurrences of FromValue to ToValue in the memory block X of
  length Count }

implementation

{$IFDEF WIN32}uses Windows;{$ENDIF}

function IntMin(A, B: Integer): Integer;
begin
  if A <= B then Result:=A else Result:=B;
end;

function IntMax(A, B: Integer): Integer;
begin
  if A >= B then Result:=A else Result:=B;
end;

function FloatMin(A, B: Float): Float;
begin
  if A <= B then Result:=A else Result:=B;
end;

function FloatMax(A, B: Float): Float;
begin
  if A >= B then Result:=A else Result:=B;
end;

function IntMinMax(Value, Floor, Ceil: Integer): Integer;
begin
  Result:=Value;
  if Result < Floor then Result:=Floor;
  if Result > Ceil then Result:=Ceil;
end;

function PhysicalMemorySize: UInt32;
{$IFDEF WIN32}
const
  MemSize: UInt32 = 0;
var
  MemoryStatus: TMemoryStatus;
begin
  if MemSize = 0 then begin
    MemoryStatus.dwLength:=SizeOf(TMemoryStatus);
    GlobalMemoryStatus({$IFDEF V_FREEPASCAL}@{$ENDIF}MemoryStatus);
    MemSize:=MemoryStatus.dwTotalPhys;
  end;
  Result:=MemSize;
{$ELSE}
begin
  Result:=MaxAvail;
{$ENDIF}
end;

function PhysicalMemoryFree: UInt32;
{$IFDEF WIN32}
var
  MemoryStatus: TMemoryStatus;
begin
  MemoryStatus.dwLength:=SizeOf(TMemoryStatus);
  GlobalMemoryStatus({$IFDEF V_FREEPASCAL}@{$ENDIF}MemoryStatus);
  Result:=MemoryStatus.dwAvailPhys;
{$ELSE}
begin
  Result:=MemAvail;
{$ENDIF}
end;

function HaveMemoryReserve: Boolean;
begin
  Result:=PhysicalMemoryFree >= MemoryReserve;
end;

function MemEqual(const X, Y; Count: Integer): Boolean;
{$IFNDEF USE_ASM}
var
  I: Integer;
begin
  for I:=0 to Count - 1 do
    if TInt8Array(X)[I] <> TInt8Array(Y)[I] then begin
      Result:=False;
      Exit;
    end;
  Result:=True;
end;
{$ELSE}
asm     // eax = @X; edx = @Y; ecx = Count
        {$IFDEF V_FREEPASCAL}
        mov      eax, X
        mov      edx, Y
        mov      ecx, Count
        {$ENDIF}
        push     esi
        push     edi
        mov      esi, edx
        mov      edi, eax
        xor      eax, eax
        or       ecx, ecx
        jl       @@Exit
        je       @@True
        mov      edx, ecx
        and      edx, 11b
        shr      ecx, 2
        jz       @@ByteOp
        repe     cmpsd
        jne      @@Exit
@@ByteOp:
        mov      ecx, edx
        repe     cmpsb
        jne      @@Exit
@@True:
        inc      eax
@@Exit:
        pop      edi
        pop      esi
@@End:
end{$IFDEF V_FREEPASCAL} ['eax','ecx','edx']{$ENDIF};
{$ENDIF}

procedure MemExchange(var X, Y; Count: Integer);
{$IFNDEF USE_ASM}
var
  I: Integer;
  T: Int8;
begin
  for I:=0 to Count - 1 do begin
    T:=TInt8Array(X)[I];
    TInt8Array(X)[I]:=TInt8Array(Y)[I];
    TInt8Array(Y)[I]:=T;
  end;
end;
{$ELSE}
asm     // eax = @X; edx = @Y; ecx = Count
        {$IFDEF V_FREEPASCAL}
        mov      eax, X
        mov      ecx, Count
        mov      edx, Y
        {$ENDIF}
        or       ecx, ecx
        jle      @@End
        push     ebx
        push     esi
        push     ecx
        xor      esi, esi
        shr      ecx, 2
        jz       @@ByteOp
@@DWordLoop:
        mov      ebx, dword ptr [eax+esi]
        xchg     dword ptr [edx+esi], ebx
        mov      dword ptr [eax+esi], ebx
        add      esi, 4
        dec      ecx
        jnz      @@DWordLoop
@@ByteOp:
        pop      ecx
        and      ecx, 11b
        jz       @@Exit
@@ByteLoop:
        mov      bl, byte ptr [eax+esi]
        xchg     byte ptr [edx+esi], bl
        mov      byte ptr [eax+esi], bl
        inc      esi
        dec      ecx
        jnz      @@ByteLoop
@@Exit:
        pop      esi
        pop      ebx
@@End:
end{$IFDEF V_FREEPASCAL} ['eax','ecx','edx']{$ENDIF};
{$ENDIF}

function FindInBuf(const SearchBytes; SearchCount: Integer; const Buffer;
  BufferLength: Integer): Integer;
{$IFNDEF USE_ASM}
var
  I, J, Limit: Integer;
begin
  if SearchCount > 0 then begin
    Limit:=BufferLength - SearchCount;
    Result:=0;
    while Result <= Limit do begin
      I:=Result;
      J:=0;
      while TInt8Array(Buffer)[I] = TInt8Array(SearchBytes)[J] do begin
        Inc(I);
        Inc(J);
        if J >= SearchCount then Exit;
      end;
      Inc(Result);
    end;
  end;
  Result:=-1;
end;
{$ELSE}
asm     // eax = @SearchBytes; edx = SearchCount; ecx = @Buffer
        {$IFDEF V_FREEPASCAL}
        mov      eax, SearchBytes
        mov      edx, SearchCount
        mov      ecx, Buffer
        {$ENDIF}
        or       edx, edx
        jle      @@IsEmpty
        push     ebx
        push     esi
        push     edi
        mov      esi, eax
        mov      edi, ecx
        mov      ecx, BufferLength
        push     edi
        dec      edx
        mov      al, [esi]
        inc      esi
        sub      ecx, edx
        jle      @@Fail
@@Loop:
        repne    scasb
        jne      @@Fail
        mov      ebx, ecx
        push     esi
        push     edi
        mov      ecx, edx
        repe     cmpsb
        pop      edi
        pop      esi
        je       @@Found
        mov      ecx, ebx
        jmp      @@Loop
@@Fail:
        pop      edx
        xor      eax, eax
        jmp      @@Exit
@@IsEmpty:
        xor      eax, eax
        jmp      @@NoWork
@@Found:
        pop      edx
        mov      eax, edi
        sub      eax, edx
@@Exit:
        pop      edi
        pop      esi
        pop      ebx
@@NoWork:
        dec      eax
end{$IFDEF V_FREEPASCAL} ['eax','ecx','edx']{$ENDIF};
{$ENDIF}

procedure FillPattern(var X; Count: Integer; const Pattern; PatternLength: Integer);
var
  I, J: Integer;
begin
  if PatternLength > 0 then begin
    J:=0;
    for I:=0 to Count - 1 do begin
      TInt8Array(X)[I]:=TInt8Array(Pattern)[J];
      Inc(J);
      if J >= PatternLength then J:=0;
    end;
  end;
end;

procedure FillValue16(var X; Value: Int16; Count: Integer);
{$IFNDEF USE_ASM}
begin
  FillPattern(X, Count * 2, Value, 2);
end;
{$ELSE}
asm     // eax = @X; dx = Value; ecx = Count
        {$IFDEF V_FREEPASCAL}
        mov      eax, X
        mov      dx, Value
        mov      ecx, Count
        {$ENDIF}
        push     edi
        mov      edi, eax
        mov      eax, edx
        shl      eax, 16
        mov      ax, dx
        shr      ecx, 1
        rep      stosd
        adc      ecx, 0
        rep      stosw
        pop      edi
end{$IFDEF V_FREEPASCAL} ['eax','ecx','edx']{$ENDIF};
{$ENDIF}

procedure FillValue32(var X; Value: Int32; Count: Integer);
{$IFNDEF USE_ASM}
begin
  FillPattern(X, Count * 4, Value, 4);
end;
{$ELSE}
asm     // eax = @X; edx = Value; ecx = Count
        {$IFDEF V_FREEPASCAL}
        mov      eax, X
        mov      edx, Value
        mov      ecx, Count
        {$ENDIF}
        push     edi
        mov      edi, eax
        mov      eax, edx
        rep      stosd
        pop      edi
end{$IFDEF V_FREEPASCAL} ['eax','ecx','edx']{$ENDIF};
{$ENDIF}

procedure FillValue64(var X; const Value; Count: Integer);
{$IFNDEF USE_ASM}
begin
  FillPattern(X, Count * 8, Value, 8);
end;
{$ELSE}
asm     // eax = @X; edx = @Value; ecx = Count
        {$IFDEF V_FREEPASCAL}
        mov      eax, X
        mov      edx, Value
        mov      ecx, Count
        {$ENDIF}
        or       ecx, ecx
        jle      @@Exit
        push     edi
        mov      edi, [edx]
        dec      ecx
        mov      edx, [edx + 4]
@@Loop: mov      [eax + ecx * 8], edi
        mov      [eax + ecx * 8 + 4], edx
        dec      ecx
        jge      @@Loop
        pop      edi
@@Exit:
end{$IFDEF V_FREEPASCAL} ['eax','ecx','edx']{$ENDIF};
{$ENDIF}

procedure FillValue80(var X; const Value; Count: Integer);
{$IFNDEF USE_ASM}
begin
  FillPattern(X, Count * 10, Value, 10);
end;
{$ELSE}
asm     // eax = @X; edx = @Value; ecx = Count
        {$IFDEF V_FREEPASCAL}
        mov      eax, X
        mov      ecx, Count
        mov      edx, Value
        {$ENDIF}
        or       ecx, ecx
        jle      @@Exit
        push     edi
        push     ebx
        mov      edi, [edx]
        mov      ebx, [edx + 4]
        mov      dx, word ptr [edx + 8]
@@Loop: mov      [eax], edi
        add      eax, 4
        mov      [eax], ebx
        add      eax, 4
        mov      word ptr [eax], dx
        add      eax, 2
        dec      ecx
        jnz      @@Loop
        pop      ebx
        pop      edi
@@Exit:
end{$IFDEF V_FREEPASCAL} ['eax','ecx','edx']{$ENDIF};
{$ENDIF}

function CountEqualToPattern(const X; Count: Integer; const Pattern;
  PatternLength: Integer): Integer;
var
  I: Integer;
begin
  Result:=0;
  for I:=0 to Count - 1 do
    if MemEqual(TInt8Array(X)[I * PatternLength], Pattern, PatternLength)
    then Inc(Result);
end;

function CountEqualToValue8(const X; Value: Int8; Count: Integer): Integer;
{$IFNDEF USE_ASM}
var
  I: Integer;
begin
  Result:=0;
  for I:=0 to Count - 1 do
    if TInt8Array(X)[I] = Value then Inc(Result);
end;
{$ELSE}
asm     // eax = @X; dl = Value; ecx = Count
        {$IFDEF V_FREEPASCAL}
        mov      eax, X
        mov      ecx, Count
        mov      dl, Value
        {$ENDIF}
        push     ebx
        push     esi
        xor      ebx, ebx
        or       ecx, ecx
        jle      @@Exit
        mov      esi, eax
        xor      eax, eax
@@Loop:
        cmp      byte ptr [esi], dl
        setz     al
        add      ebx, eax
        inc      esi
        dec      ecx
        jnz      @@Loop
@@Exit:
        mov      eax, ebx
        pop      esi
        pop      ebx
end{$IFDEF V_FREEPASCAL} ['eax','ecx','edx']{$ENDIF};
{$ENDIF}

function CountEqualToValue16(const X; Value: Int16; Count: Integer): Integer;
{$IFNDEF USE_ASM}
var
  I: Integer;
begin
  Result:=0;
  for I:=0 to Count - 1 do
    if TInt16Array(X)[I] = Value then Inc(Result);
end;
{$ELSE}
asm     // eax = @X; dx = Value; ecx = Count
        {$IFDEF V_FREEPASCAL}
        mov      eax, X
        mov      ecx, Count
        mov      dx, Value
        {$ENDIF}
        push     ebx
        push     esi
        xor      ebx, ebx
        or       ecx, ecx
        jle      @@Exit
        mov      esi, eax
        xor      eax, eax
@@Loop:
        cmp      word ptr [esi], dx
        setz     al
        add      ebx, eax
        add      esi, 2
        dec      ecx
        jnz      @@Loop
@@Exit:
        mov      eax, ebx
        pop      esi
        pop      ebx
end{$IFDEF V_FREEPASCAL} ['eax','ecx','edx']{$ENDIF};
{$ENDIF}

function CountEqualToValue32(const X; Value: Int32; Count: Integer): Integer;
{$IFNDEF USE_ASM}
var
  I: Integer;
begin
  Result:=0;
  for I:=0 to Count - 1 do
    if TInt32Array(X)[I] = Value then Inc(Result);
end;
{$ELSE}
asm     // eax = @X; edx = Value; ecx = Count
        {$IFDEF V_FREEPASCAL}
        mov      eax, X
        mov      ecx, Count
        mov      edx, Value
        {$ENDIF}
        push     ebx
        push     esi
        xor      ebx, ebx
        or       ecx, ecx
        jle      @@Exit
        mov      esi, eax
        xor      eax, eax
@@Loop:
        cmp      [esi], edx
        setz     al
        add      ebx, eax
        add      esi, 4
        dec      ecx
        jnz      @@Loop
@@Exit:
        mov      eax, ebx
        pop      esi
        pop      ebx
end{$IFDEF V_FREEPASCAL} ['eax','ecx','edx']{$ENDIF};
{$ENDIF}

function CountEqualToValue64(const X; const Value; Count: Integer): Integer;
var
  I: Integer;
begin
  Result:=0;
  for I:=0 to Count - 1 do
    if TFloat64Array(X)[I] = Float64(Value) then Inc(Result);
end;

function CountEqualToValue80(const X; const Value; Count: Integer): Integer;
var
  I: Integer;
begin
  Result:=0;
  for I:=0 to Count - 1 do
    if TFloat80Array(X)[I] = Float80(Value) then Inc(Result);
end;

function IndexOfPattern(const X; Count: Integer; const Pattern;
  PatternLength: Integer): Integer;
begin
  Result:=0;
  while Result < Count do
    if MemEqual(TInt8Array(X)[Result * PatternLength], Pattern, PatternLength)
    then Exit
    else Inc(Result);
  Result:=-1;
end;

function IndexOfValue8(const X; Value: Int8; Count: Integer): Integer;
{$IFNDEF USE_ASM}
begin
  Result:=0;
  while Result < Count do
    if TInt8Array(X)[Result] <> Value then Inc(Result)
    else Exit;
  Result:=-1;
end;
{$ELSE}
asm     // eax = @X; dl = Value; ecx = Count
        {$IFDEF V_FREEPASCAL}
        mov      eax, X
        mov      ecx, Count
        mov      dl, Value
        {$ENDIF}
        push     edi
        push     ebx
        or       ecx, ecx
        jle      @@NotFound
        mov      edi, eax
        mov      ebx, eax
        mov      eax, edx
        repne    scasb
        jne      @@NotFound
        mov      eax, edi
        inc      ebx
        sub      eax, ebx
        jmp      @@Exit
@@NotFound:
        mov      eax, -1
@@Exit:
        pop      ebx
        pop      edi
end{$IFDEF V_FREEPASCAL} ['eax','ecx','edx']{$ENDIF};
{$ENDIF}

function IndexOfValue16(const X; Value: Int16; Count: Integer): Integer;
{$IFNDEF USE_ASM}
begin
  Result:=0;
  while Result < Count do
    if TInt16Array(X)[Result] <> Value then Inc(Result)
    else Exit;
  Result:=-1;
end;
{$ELSE}
asm     // eax = @X; dx = Value; ecx = Count
        {$IFDEF V_FREEPASCAL}
        mov      eax, X
        mov      ecx, Count
        mov      dx, Value
        {$ENDIF}
        push     edi
        push     ebx
        or       ecx, ecx
        jle      @@NotFound
        mov      edi, eax
        mov      ebx, eax
        mov      eax, edx
        repne    scasw
        jne      @@NotFound
        mov      eax, edi
        add      ebx, 2
        sub      eax, ebx
        shr      eax, 1
        jmp      @@Exit
@@NotFound:
        mov      eax, -1
@@Exit:
        pop      ebx
        pop      edi
end{$IFDEF V_FREEPASCAL} ['eax','ecx','edx']{$ENDIF};
{$ENDIF}

function IndexOfValue32(const X; Value: Int32; Count: Integer): Integer;
{$IFNDEF USE_ASM}
begin
  Result:=0;
  while Result < Count do
    if TInt32Array(X)[Result] <> Value then Inc(Result)
    else Exit;
  Result:=-1;
end;
{$ELSE}
asm     // eax = @X; edx = Value; ecx = Count
        {$IFDEF V_FREEPASCAL}
        mov      eax, X
        mov      ecx, Count
        mov      edx, Value
        {$ENDIF}
        push     edi
        push     ebx
        or       ecx, ecx
        jle      @@NotFound
        mov      edi, eax
        mov      ebx, eax
        mov      eax, edx
        repne    scasd
        jne      @@NotFound
        mov      eax, edi
        add      ebx, 4
        sub      eax, ebx
        shr      eax, 2
        jmp      @@Exit
@@NotFound:
        mov      eax, -1
@@Exit:
        pop      ebx
        pop      edi
end{$IFDEF V_FREEPASCAL} ['eax','ecx','edx']{$ENDIF};
{$ENDIF}

function IndexOfValue64(const X; const Value; Count: Integer): Integer;
begin
  Result:=0;
  while Result < Count do
    if TFloat64Array(X)[Result] <> Float64(Value) then Inc(Result)
    else Exit;
  Result:=-1;
end;

function IndexOfValue80(const X; const Value; Count: Integer): Integer;
begin
  Result:=0;
  while Result < Count do
    if TFloat80Array(X)[Result] <> Float80(Value) then Inc(Result)
    else Exit;
  Result:=-1;
end;

function LastIndexOfPattern(const X; Count: Integer; const Pattern;
  PatternLength: Integer): Integer;
begin
  Result:=Count - 1;
  while Result >= 0 do
    if MemEqual(TInt8Array(X)[Result * PatternLength], Pattern, PatternLength)
    then Exit
    else Dec(Result);
end;

function LastIndexOfValue8(const X; Value: Int8; Count: Integer): Integer;
{$IFNDEF USE_ASM}
begin
  Result:=Count - 1;
  while Result >= 0 do
    if TInt8Array(X)[Result] <> Value then Dec(Result)
    else Exit;
end;
{$ELSE}
asm     // eax = @X; dl = Value; ecx = Count
        {$IFDEF V_FREEPASCAL}
        mov      eax, X
        mov      ecx, Count
        mov      dl, Value
        {$ENDIF}
        push     edi
        push     ebx
        or       ecx, ecx
        jle      @@NotFound
        mov      edi, eax
        mov      ebx, eax
        add      edi, ecx
        mov      eax, edx
        dec      edi
        std
        repne    scasb
        cld
        jne      @@NotFound
        mov      eax, edi
        dec      ebx
        sub      eax, ebx
        jmp      @@Exit
@@NotFound:
        mov      eax, -1
@@Exit:
        pop      ebx
        pop      edi
end{$IFDEF V_FREEPASCAL} ['eax','ecx','edx']{$ENDIF};
{$ENDIF}

function LastIndexOfValue16(const X; Value: Int16; Count: Integer): Integer;
{$IFNDEF USE_ASM}
begin
  Result:=Count - 1;
  while Result >= 0 do
    if TInt16Array(X)[Result] <> Value then Dec(Result)
    else Exit;
end;
{$ELSE}
asm     // eax = @X; dx = Value; ecx = Count
        {$IFDEF V_FREEPASCAL}
        mov      eax, X
        mov      ecx, Count
        mov      dx, Value
        {$ENDIF}
        push     edi
        push     ebx
        or       ecx, ecx
        jle      @@NotFound
        mov      edi, eax
        mov      ebx, eax
        mov      eax, ecx
        dec      eax
        shl      eax, 1
        add      edi, eax
        mov      eax, edx
        std
        repne    scasw
        cld
        jne      @@NotFound
        mov      eax, edi
        sub      ebx, 2
        sub      eax, ebx
        shr      eax, 1
        jmp      @@Exit
@@NotFound:
        mov      eax, -1
@@Exit:
        pop      ebx
        pop      edi
end{$IFDEF V_FREEPASCAL} ['eax','ecx','edx']{$ENDIF};
{$ENDIF}

function LastIndexOfValue32(const X; Value: Int32; Count: Integer): Integer;
{$IFNDEF USE_ASM}
begin
  Result:=Count - 1;
  while Result >= 0 do
    if TInt32Array(X)[Result] <> Value then Dec(Result)
    else Exit;
end;
{$ELSE}
asm     // eax = @X; edx = Value; ecx = Count
        {$IFDEF V_FREEPASCAL}
        mov      eax, X
        mov      ecx, Count
        mov      edx, Value
        {$ENDIF}
        push     edi
        push     ebx
        or       ecx, ecx
        jle      @@NotFound
        mov      edi, eax
        mov      ebx, eax
        mov      eax, ecx
        dec      eax
        shl      eax, 2
        add      edi, eax
        mov      eax, edx
        std
        repne    scasd
        cld
        jne      @@NotFound
        mov      eax, edi
        sub      ebx, 4
        sub      eax, ebx
        shr      eax, 2
        jmp      @@Exit
@@NotFound:
        mov      eax, -1
@@Exit:
        pop      ebx
        pop      edi
end{$IFDEF V_FREEPASCAL} ['eax','ecx','edx']{$ENDIF};
{$ENDIF}

function LastIndexOfValue64(const X; const Value; Count: Integer): Integer;
begin
  Result:=Count - 1;
  while Result >= 0 do
    if TFloat64Array(X)[Result] <> Float64(Value) then Dec(Result)
    else Exit;
end;

function LastIndexOfValue80(const X; const Value; Count: Integer): Integer;
begin
  Result:=Count - 1;
  while Result >= 0 do
    if TFloat80Array(X)[Result] <> Float80(Value) then Dec(Result)
    else Exit;
end;

function ReplaceValue8(const X; FromValue, ToValue: Int8; Count: Integer): Integer;
var
  P, Limit: PChar;
begin
  Result:=0;
  P:=@X;
  Limit:=P + Count;
  while P < Limit do begin
    if P^ = Char(FromValue) then begin
      P^:=Char(ToValue);
      Inc(Result);
    end;
    Inc(P);
  end;
end;

end.
