{ Version 010608. Copyright  Alexey A.Chernobaev, 1996-2001 }

unit SPDic;
{
   string-pointer.

  String-pointer dictionary.
}

interface

{$I VCheck.inc}

uses
  SysUtils, ExtType, Pointerv, StrLst, VStream, VectErr;

type
  TDicKey = String;
  TDicData = Pointer;

  {$DEFINE USER_COMPARE_OBJECTS}
  {$I Dic.def}

  TStrPtrDic = class(TDic)
  protected
    function CMP(const a, b: TTreeData): Integer;
    procedure WriteItem(VStream: TVStream; const Item: TTreeData); override;
    function ReadItem(VStream: TVStream): TTreeData; override;
    procedure FreeItem(const Item: TTreeData);
  public
    constructor Create;
    class function Compare(const S1, S2: String): Integer; virtual;
    {  - CMP    }
    { used by the function method CMP for comparing strings }
    procedure CopyToStrLst(S: TStrLst);
    {   (    Compare)   
         S }
    { copies sorted (according to Compare method) keys from the dictionary to
      the string list S }
    procedure CopyToStrLstWithData(S: TStrLst; DataVector: TPointerVector);
    {  CopyToStrLst,       
      (DataList[I]:=Data(S[I]) }
    { analog of CopyToStrLst which also returns data linked with the keys
      (DataList[I]:=Data(S[I]) }
    procedure CopyToDataVector(DataVector: TPointerVector);
    {  ,   ,  DataVector }
    { copies data linked with the keys to DataVector }
    procedure FreeAndDelete(const AKey: TDicKey);
    {       AKey,    TObject }
    { frees and deletes element with the key AKey interpreting it as TObject }
    procedure FreeItems;
    {   ,    TObject }
    { frees all elements interpreting them as TObject }
    procedure DebugWriteItem(const Item: TTreeData);
    procedure DebugWrite;
  end;

  TStrPtrDicClass = class of TStrPtrDic;

  TCaseSensStrPtrDic = class(TStrPtrDic)
    class function Compare(const S1, S2: String): Integer; override;
  end;

  TCaseSensStrPtrDicClass = class of TCaseSensStrPtrDic;

  TStrPtrDicIterator = TDicIterator;

const
  SKeyNotFound = 'Key ''%s'' not found';

implementation

{$I Dic.imp}

{ TStrPtrDic }

constructor TStrPtrDic.Create;
begin
  inherited Create(CMP);
end;

function TStrPtrDic.CMP(const a, b: TTreeData): Integer;
begin
  Result:=Compare(a.Key, b.Key);
end;

procedure TStrPtrDic.WriteItem(VStream: TVStream; const Item: TTreeData);
begin
  VStream.WriteString(Item.Key);
  VStream.WriteInt32(Int32(Item.Data));
end;

function TStrPtrDic.ReadItem(VStream: TVStream): TTreeData;
begin
  Result.Key:=VStream.ReadString;
  Result.Data:=Pointer(VStream.ReadInt32);
end;

class function TStrPtrDic.Compare(const S1, S2: String): Integer;
begin
  Result:=AnsiCompareText(S1, S2);
end;

procedure TStrPtrDic.CopyToStrLst(S: TStrLst);
var
  I: Integer;
  It: TStrPtrDicIterator;
begin
  S.Count:=Count;
  It:=TStrPtrDicIterator.Create(Self);
  try
    I:=0;
    while not It.Eof do begin
      S.Items[I]:=It.Data.Key;
      Inc(I);
      It.Next;
    end;
  finally
    It.Free;
  end;
end;

procedure TStrPtrDic.CopyToStrLstWithData(S: TStrLst; DataVector: TPointerVector);
var
  I: Integer;
  It: TStrPtrDicIterator;
begin
  I:=Count;
  S.Count:=I;
  DataVector.Count:=I;
  It:=TStrPtrDicIterator.Create(Self);
  try
    I:=0;
    while not It.Eof do begin
      S.Items[I]:=It.Data.Key;
      DataVector[I]:=It.Data.Data;
      Inc(I);
      It.Next;
    end;
  finally
    It.Free;
  end;
end;

procedure TStrPtrDic.CopyToDataVector(DataVector: TPointerVector);
var
  I: Integer;
  It: TStrPtrDicIterator;
begin
  I:=Count;
  DataVector.Count:=I;
  It:=TStrPtrDicIterator.Create(Self);
  try
    I:=0;
    while not It.Eof do begin
      DataVector[I]:=It.Data.Data;
      Inc(I);
      It.Next;
    end;
  finally
    It.Free;
  end;
end;

procedure TStrPtrDic.FreeAndDelete(const AKey: TDicKey);
var
  TreeData: TTreeData;
  Node: PNode;
begin
  TreeData.Key:=AKey;
  Node:=FindNode(TreeData);
  if Node <> nil then begin
    TObject(Node^.data.Data).Free;
    DeleteNode(Node);
  end
  else
    raise EDicError.CreateFmt(SKeyNotFound, [AKey]);
end;

procedure TStrPtrDic.FreeItem(const Item: TTreeData);
begin
  TObject(Item.Data).Free;
end;

procedure TStrPtrDic.FreeItems;
begin
  if Self <> nil then Traversal(FreeItem);
end;

procedure TStrPtrDic.DebugWriteItem(const Item: TTreeData);
begin
  writeln(Item.Key, ' ', IntToHex(Int32(Item.Data), 8));
end;

procedure TStrPtrDic.DebugWrite;
begin
  UpwardTraversal(DebugWriteItem);
end;

{ TCaseSensStrPtrDic }

class function TCaseSensStrPtrDic.Compare(const S1, S2: String): Integer;
begin
  Result:=AnsiCompareStr(S1, S2);
end;

end.
