{ Version 010318. Copyright  Alexey A.Chernobaev, 1996-2001 }

unit StrLst;

interface

{$I VCheck.inc}

uses
  SysUtils, ExtType, Vectors, Pointerv, VStream, VTxtStrm, VectStr, VectErr;

type
  {       ( "Compare" 
    AnsiCompareText) }
  { case-insensitive string list ("Compare" method uses AnsiCompareText) }
  TStrLst = class(TPointerVector)
  protected
    procedure FreeItems(FromIndex: Integer);
    procedure SetCount(ACount: Integer); override;
    function GetValue(I: Integer): String;
    procedure SetValue(I: Integer; const Value: String);
    function GetCommaText: String;
    function GetCommaText1: String;
    procedure FSetCommaText(const Value: String; QuoteChar: Char);
    procedure SetCommaText(const Value: String);
    procedure SetCommaText1(const Value: String);
  public
    function CreateCompatibleVector: TPointerVector; override;
    destructor Destroy; override;
    procedure WriteToStream(VStream: TVStream); override;
    procedure ReadFromStream(VStream: TVStream); override;
    procedure WriteToTextStream(TextStream: TTextStream);
    procedure ReadFromTextStream(TextStream: TTextStream);
    procedure Assign(Source: TVector); override;
    function EqualTo(V: TVector): Bool; override;
    procedure Delete(I: Integer); override;
    procedure DeleteRange(I, ACount: Integer); override;
    procedure GetUntyped(I: Integer; var Result); override;
    procedure SetUntyped(I: Integer; const Value); override;
    class function CompareStrings(const S1, S2: String): Int32; virtual;
    function Compare(I: Integer; const V): Int32; override;
    procedure SetToDefault; override;
    procedure SetItems(Values: array of String);
    {      Values (Count:=High(Values) + 1) }
    { sets the list elements to Values (Count:=High(Values) + 1) }
    procedure Insert(I: Integer; const Value: String); virtual;
    {   Value   I }
    { inserts Value in the position I }
    function Add(const Value: String): Integer; virtual;
    {          (Count - 1) }
    { appends Value to the end of the list and returns it's index (Count - 1) }
    procedure Move(CurIndex, NewIndex: Integer); virtual;
    {    CurIndex  NewIndex }
    { moves the element from the position CurIndex to NewIndex }
    function IndexFrom(I: Integer; const Value: String): Integer; virtual;
    {   ,   I,   Value  ,
       -1,      }
    { returns the index of the first occurrence of Value in the list beginning
      from I or -1 if there's no such occurrence }
    function IndexOf(const Value: String): Integer;
    { IndexOf(Value) = IndexFrom(0, Value) }
    function LastIndexFrom(I: Integer; const Value: String): Integer; virtual;
    {      Value  , 
        I,  -1,     }
    { returns the index of the last occurrence of Value in the list which is not
      greater then I or -1 if there's no such occurrence }
    function LastIndexOf(const Value: String): Integer;
    { LastIndexOf(Value) = LastIndexFrom(Count - 1, Value) }
    function Remove(const Value: String): Integer;
    {    Value  ,    Delete 
         ,  -1,  Value   }
    { searches for the first occurrence of Value in the list, deletes it with
      Delete and returns the index of the deleted value or -1 if Value wasn't
      found }
    function RemoveLast(const Value: String): Integer;
    {    Value  ,    Delete 
         ,  -1,  Value   }
    { searches for the last occurrence of Value in the list, deletes it with
      Delete and returns the index of the deleted value or -1 if Value wasn't
      found }
    function RemoveFrom(I: Integer; const Value: String): Integer;
    {  ,   I,  Value  ,   
      Delete     ,  -1,  Value
        }
    { searches for the first occurrence of Value in the list beginning from I,
      deletes it with Delete and returns the index of the deleted value or -1 if
      Value wasn't found }
    function RemoveLastFrom(I: Integer; const Value: String): Integer;
    {  ,    I,  Value     
       Delete,    ,  -1,  Value
        }
    { searches for the last occurrence of Value in the list which is not greater
      then I, deletes it with Delete and returns the index of the deleted value
      or -1 if Value wasn't found }
    function NumberOfValues(const Value: String): Integer;
    {   ,  Value }
    { returns the number of elements equal to Value }
    function FindInSortedRange(const Value: String; L, H: Integer): Integer;
    {    Value    
      ,    L   H;   
       ,  -1,     }
    { searches for the Value in the sorted (ascending) list dichotomically
      from the index L to H; returns the minimum index of Value or -1 if Value
      wasn't found }
    function FindInSorted(const Value: String): Integer;
    {   Value      ;
          ,  -1,  
        }
    { searches for the Value in the sorted (ascending) list dichotomically;
      returns the minimum index of Value or -1 if Value wasn't found }
    function FindInsertPosition(const Value: String; L, H: Integer;
      var Index: Integer): Bool;
    {   Value      ,
         L   H;  True,   
      (  Index     ), 
       False (  Index ,    Value,
         ) }
    { searches for the Value in the sorted (ascending) list dichotomically
      from the index L to H; returns True if Value was found (in such case Index
      is equal to the minimum index of Value), otherwise returns False (in such
      case Index is equal to the position where Value can be inserted so that
      the list remains sorted) }
    function Find(const Value: String; var Index: Integer): Bool;
    {  FindInsertPosition   }
    { analog of FindInsertPosition for the whole list }
    procedure AddStrings(List: TStrLst);
    {   List }
    { adds strings List }
    function Last: String;
    {     (    ) }
    { returns the last element of the list (which must be non-empty) }
    function Pop: String;
    {     (    )
         (..     ) }
    { returns the last element of the list (which must be non-empty) and removes
      it (i.e. decreases the length of the list by one) }
    property CommaText: String read GetCommaText write SetCommaText;
    {     ,     ,
              }
    { presents the list as a single comma-delimited string where the list
      elements are separated with commas and enclosed in the double quotes }
    property CommaText1: String read GetCommaText1 write SetCommaText1;
    {     ,     ,
             ,   
       ,    ,     
      (. CheckText   VectStr) }
    { presents the list as a single comma-delimited string where the list
      elements are separated with commas and enclosed in the single quotes if
      they contain characters other then the Latin letters, digits and several
      other characters (see ChectText in the unit VectStr) }
    property Items[I: Integer]: String read GetValue write SetValue; {$IFDEF V_32}default;{$ENDIF}
    procedure DebugWrite;
    {  ;      
      Win32-      AllocConsole }
    { debug write; to use in Win32 GUI applications it's necessary to create
      console with AllocConsole }
  end;

  TStrLstClass = class of TStrLst;

  {       ( "Compare" 
    AnsiCompareStr) }
  { case-sensitive string list ("Compare" method uses AnsiCompareStr) }
  TCaseSensStrLst = class(TStrLst)
    function CreateCompatibleVector: TPointerVector; override;
    class function CompareStrings(const S1, S2: String): Int32; override;
  end;

  TCaseSensStrLstClass = class of TCaseSensStrLst;

  {       ( "Compare" 
       "<"  ">") }
  { case-sensitive string list ("Compare" method uses built-in compare operators
    "<" and ">") }
  TExactStrLst = class(TStrLst)
    function CreateCompatibleVector: TPointerVector; override;
    class function CompareStrings(const S1, S2: String): Int32; override;
  end;

  TExactStrLstClass = class of TCaseSensStrLst;

  TSortedStrLst = class(TStrLst)
    function CreateCompatibleVector: TPointerVector; override;
    procedure Insert(I: Integer; const Value: String); override;
    function Add(const Value: String): Integer; override;
    procedure Move(CurIndex, NewIndex: Integer); override;
    function IndexFrom(I: Integer; const Value: String): Integer; override;
    function LastIndexFrom(I: Integer; const Value: String): Integer; override;
  end;

  TSortedStrLstClass = class of TSortedStrLst;

  TCaseSensSortedStrLst = class(TSortedStrLst)
    function CreateCompatibleVector: TPointerVector; override;
    class function CompareStrings(const S1, S2: String): Int32; override;
  end;

  TCaseSensSortedStrLstClass = class of TCaseSensSortedStrLst;

implementation

{ TStrLst }

procedure TStrLst.FreeItems(FromIndex: Integer);
var
  I: Integer;
begin
  for I:=FromIndex to Count - 1 do Items[I]:='';
end;

procedure TStrLst.SetCount(ACount: Integer);
begin
  if ACount < Count then FreeItems(ACount);
  inherited SetCount(ACount);
end;

function TStrLst.GetValue(I: Integer): String;
var
  P: PVString;
begin
  {$IFDEF CHECK_VECTORS}
  if (I < 0) or (I >= Count) then Error(SRangeError, [I]);
  {$ENDIF}
  P:=PPointerArray(FItems)^[I];
  if P <> nil then Result:=P^ else Result:='';
end;

procedure TStrLst.SetValue(I: Integer; const Value: String);
begin
  {$IFDEF CHECK_VECTORS}
  if (I < 0) or (I >= Count) then Error(SRangeError, [I]);
  {$ENDIF}
  DisposeStr(PPointerArray(FItems)^[I]);
  PPointerArray(FItems)^[I]:=NewStr(Value);
end;

function TStrLst.GetCommaText: String;
var
  I: Integer;
begin
  Result:='';
  if Count > 0 then begin
    Result:=StringToLiteral2(Items[0]);
    for I:=1 to Count - 1 do
      Result:=Result + ',' + StringToLiteral2(Items[I]);
  end;
end;

function TStrLst.GetCommaText1: String;
var
  I: Integer;
begin
  Result:='';
  if Count > 0 then begin
    Result:=TextToLiteral(Items[0]);
    for I:=1 to Count - 1 do
      Result:=Result + ', ' + TextToLiteral(Items[I]);
  end;
end;

procedure TStrLst.FSetCommaText(const Value: String; QuoteChar: Char);
var
  I: Integer;
  C: Char;
  Quote: Bool;
  S: String;
begin
  Clear;
  S:='';
  Quote:=False;
  for I:=1 to Length(Value) do begin
    C:=Value[I];
    if Quote or (C <> ',') then begin
      if C = QuoteChar then Quote:=not Quote;
      S:=S + C;
    end
    else begin
      Add(LiteralToString(S));
      S:='';
    end;
  end;
  if S <> '' then Add(LiteralToString(S));
end;

procedure TStrLst.SetCommaText(const Value: String);
begin
  FSetCommaText(Value, '"');
end;

procedure TStrLst.SetCommaText1(const Value: String);
begin
  FSetCommaText(Value, '''');
end;

function TStrLst.CreateCompatibleVector: TPointerVector;
begin
  Result:=TStrLst.Create;
end;

destructor TStrLst.Destroy;
begin
  SetToDefault;
  inherited Destroy;
end;

procedure TStrLst.WriteToStream(VStream: TVStream);
var
  I: Integer;
begin
  VStream.WriteInt32(FCount);
  for I:=0 to FCount - 1 do VStream.WriteString(Items[I]);
end;

procedure TStrLst.ReadFromStream(VStream: TVStream);
var
  I: Integer;
begin
  Clear;
  for I:=0 to VStream.ReadInt32 - 1 do Add(VStream.ReadString);
end;

procedure TStrLst.WriteToTextStream(TextStream: TTextStream);
var
  I: Integer;
begin
  for I:=0 to FCount - 1 do TextStream.WriteString(Items[I]);
end;

procedure TStrLst.ReadFromTextStream(TextStream: TTextStream);
begin
  Clear;
  while not TextStream.Eof do Add(TextStream.ReadString);
end;

procedure TStrLst.Delete(I: Integer);
begin
  Items[I]:='';
  inherited Delete(I);
end;

procedure TStrLst.DeleteRange(I, ACount: Integer);
var
  J: Integer;
begin
  for J:=I to I + ACount - 1 do
    Items[J]:='';
  inherited DeleteRange(I, ACount);
end;

procedure TStrLst.GetUntyped(I: Integer; var Result);
begin
  PVString(Result):=inherited GetValue(I);
end;

procedure TStrLst.SetUntyped(I: Integer; const Value);
begin
  Items[I]:=PVString(Value)^;
end;

class function TStrLst.CompareStrings(const S1, S2: String): Int32;
begin
  Result:=AnsiCompareText(S1, S2);
end;

function TStrLst.Compare(I: Integer; const V): Int32;
var
  S: String;
begin
  if PVString(V) <> nil then
    S:=PVString(V)^
  else
    S:='';
  Result:=CompareStrings(Items[I], S);
end;

procedure TStrLst.SetToDefault;
begin
  FreeItems(0);
end;

procedure TStrLst.SetItems(Values: array of String);
var
  I: Integer;
begin
  Count:=High(Values) + 1;
  for I:=0 to High(Values) do
    Items[I]:=Values[I];
end;

procedure TStrLst.Insert(I: Integer; const Value: String);
var
  P: PVString;
begin
  P:=NewStr(Value);
  try
    inherited Insert(I, P);
  except
    DisposeStr(P);
    raise;
  end; 
end;

function TStrLst.Add(const Value: String): Integer;
begin
  Result:=Count;
  Insert(Result, Value);
end;

procedure TStrLst.Move(CurIndex, NewIndex: Integer);
var
  T: String;
begin
  if CurIndex <> NewIndex then begin
    T:=Items[CurIndex];
    Delete(CurIndex);
    Insert(NewIndex, T);
  end;
end;

function TStrLst.IndexFrom(I: Integer; const Value: String): Integer;
var
  ACount: Integer;
  P: PVString;
begin
  Result:=I;
  ACount:=Count;
  P:=NewStr(Value);
  try
    while Result < ACount do
      if Compare(Result, P) = 0 then Exit else Inc(Result);
    Result:=-1;
  finally
    DisposeStr(P);
  end;
end;

function TStrLst.IndexOf(const Value: String): Integer;
begin
  Result:=IndexFrom(0, Value);
end;

function TStrLst.LastIndexFrom(I: Integer; const Value: String): Integer;
var
  P: PVString;
begin
  P:=NewStr(Value);
  try
    Result:=I;
    while Result >= 0 do
      if Compare(Result, P) = 0 then Exit else Dec(Result);
  finally
    DisposeStr(P);
  end;
end;

function TStrLst.LastIndexOf(const Value: String): Integer;
begin
  Result:=LastIndexFrom(Count - 1, Value);
end;

function TStrLst.Remove(const Value: String): Integer;
begin
  Result:=IndexOf(Value);
  if Result >= 0 then Delete(Result);
end;

function TStrLst.RemoveLast(const Value: String): Integer;
begin
  Result:=LastIndexOf(Value);
  if Result >= 0 then Delete(Result);
end;

function TStrLst.RemoveFrom(I: Integer; const Value: String): Integer;
begin
  Result:=IndexFrom(I, Value);
  if Result >= 0 then Delete(Result);
end;

function TStrLst.RemoveLastFrom(I: Integer; const Value: String): Integer;
begin
  Result:=LastIndexFrom(I, Value);
  if Result >= 0 then Delete(Result);
end;

function TStrLst.NumberOfValues(const Value: String): Integer;
var
  I: Integer;
  P: PVString;
begin
  P:=NewStr(Value);
  try
    Result:=0;
    for I:=0 to Count - 1 do
      if Compare(I, P) = 0 then Inc(Result);
  finally
    DisposeStr(P);
  end;
end;

function TStrLst.FindInsertPosition(const Value: String; L, H: Integer;
  var Index: Integer): Bool;
var
  I, C: Integer;
  P: PVString;
begin
  Result:=False;
  P:=NewStr(Value);
  try
    while L <= H do begin
      I:=(L + H) shr 1;
      C:=Compare(I, P);
      if C < 0 then L:=I + 1
      else begin
        H:=I - 1;
        if C = 0 then begin
          Result:=True;
          L:=I;
        end;
      end;
    end;
  finally
    DisposeStr(P);
  end;
  Index:=L;
end;

function TStrLst.Find(const Value: String; var Index: Integer): Bool;
begin
  Result:=FindInsertPosition(Value, 0, FCount - 1, Index);
end;

function TStrLst.FindInSortedRange(const Value: String; L, H: Integer): Integer;
begin
  if not FindInsertPosition(Value, L, H, Result) then Result:=-1;
end;

function TStrLst.FindInSorted(const Value: String): Integer;
begin
  Result:=FindInSortedRange(Value, 0, FCount - 1);
end;

procedure TStrLst.Assign(Source: TVector);
var
  I: Integer;
begin
  if Source is TStrLst then begin
    Count:=Source.Count;
    for I:=0 to Count - 1 do
      Items[I]:=TStrLst(Source).Items[I];
  end
  else
    Error(SAssignError, [0]);
end;

{$IFDEF NOWARN}{$WARNINGS OFF}{$ENDIF}
function TStrLst.EqualTo(V: TVector): Bool;
var
  I: Integer;
begin
  if V is TStrLst then begin
    if FCount = V.Count then
      for I:=0 to FCount - 1 do
        if Compare(I, PPointerArray(TStrLst(V).FItems)^[I]) <> 0 then begin
          Result:=False;
          Exit;
        end;
    Result:=True;
  end
  else
    Error(SIncompatibleClasses, [0]);
end;
{$IFDEF NOWARN}{$WARNINGS ON}{$ENDIF}

procedure TStrLst.AddStrings(List: TStrLst);
var
  I: Integer;
begin
  for I:=0 to List.Count - 1 do Add(List.Items[I]);
end;

function TStrLst.Last: String;
begin
  Result:=Items[Count - 1];
end;

function TStrLst.Pop: String;
var
  N: Integer;
begin
  N:=Count - 1;
  Result:=Items[N];
  Count:=N;
end;

procedure TStrLst.DebugWrite;
var
  I, N: Integer;
begin
  N:=FCount - 1;
  for I:=0 to N do begin
    write(Items[I]);
    if I < N then write(', ') else writeln;
  end;
end;

{ TCaseSensStrLst }

function TCaseSensStrLst.CreateCompatibleVector: TPointerVector;
begin
  Result:=TCaseSensStrLst.Create;
end;

class function TCaseSensStrLst.CompareStrings(const S1, S2: String): Int32;
begin
  Result:=AnsiCompareStr(S1, S2);
end;

{ TExactStrLst }

function TExactStrLst.CreateCompatibleVector: TPointerVector;
begin
  Result:=TExactStrLst.Create;
end;

class function TExactStrLst.CompareStrings(const S1, S2: String): Int32; 
begin
  if S1 < S2 then Result:=-1
  else if S1 > S2 then Result:=1
  else Result:=0;
end;

{ TSortedStrLst }

function TSortedStrLst.CreateCompatibleVector: TPointerVector;
begin
  Result:=TSortedStrLst.Create;
end;

procedure TSortedStrLst.Insert(I: Integer; const Value: String);
begin
  Error(SMethodNotApplicable, [0]);
end;

function TSortedStrLst.Add(const Value: String): Integer;
begin
  if FindInsertPosition(Value, 0, FCount - 1, Result) then
    Error(SDuplicateError, [0])
  else
    inherited Insert(Result, Value);
end;

procedure TSortedStrLst.Move(CurIndex, NewIndex: Integer);
begin
  Error(SMethodNotApplicable, [0]);
end;

function TSortedStrLst.IndexFrom(I: Integer; const Value: String): Integer;
begin
  Result:=FindInSorted(Value);
end;

function TSortedStrLst.LastIndexFrom(I: Integer; const Value: String): Integer;
begin
  Result:=FindInSorted(Value);
end;

{ TCaseSensSortedStrLst }

function TCaseSensSortedStrLst.CreateCompatibleVector: TPointerVector;
begin
  Result:=TCaseSensSortedStrLst.Create;
end;

class function TCaseSensSortedStrLst.CompareStrings(const S1, S2: String): Int32;
begin
  Result:=AnsiCompareStr(S1, S2);
end;

end.
