{ Version 010808. Copyright  Alexey A.Chernobaev, 1996-2001 }

unit VFileSys;

interface

{$I VCheck.inc}

uses
  Windows, SysUtils, StrLst;

function LockFileWrite(const FileName: String): THandle;
{    FileName;  Handle    (< 0) }

function UnlockFileWrite(const Handle: THandle): Boolean;
{     }

function GetLongFileName(const Name: String): String;
{  "" ,   ""  
      ,      ; 
     ""  }
{ returns the long name corresponding to the specified short file or directory
  name or the empty string if the file or directory are not found; it's legal to
  pass long names to the function }

function DirectoryExists(const Name: String): Boolean;
{ ,     }
{ checks whether the specified directory exists }

function ForceDirectories(Dir: String): Boolean;
{        ,   
    }
{ creates all the directories along the specified directory path if they do not
  already exist}

function ForceDeleteFileOrDir(const Name: String): Boolean;
{    ,       READONLY }
{ deletes the specified file or directory even if it has READONLY attribute set }

procedure ForceDeleteFiles(List: TStrLst);
{  ForceDeleteFileOrDir    List }
{ applies ForceDeleteFileOrDir to all elements of List }

function ExcludeFileExt(const Name: String): String;
{      }
{ excludes extension from the given file name }

function GetFileList(List: TStrLst; const Path, Mask: String; SubDirs: Boolean): Integer;
{   Path (    Path  SubDirs = True)
     List    ,   Mask;
      }
{ scans the directory Path (and all it's subdirectories if SubDirs = True) and
  writes full names of files matching Mask to the List; returns the number of
  the files found }

function GetFileListEx(List: TStrLst; Path, Mask: String; SubDirs: Boolean): Boolean;
{  Path   ,    ,  GetFileList,
    GetFileList(List, ExtractFilePath(Path), ExtractFileName(Path),
  SubDirs);  True,     Path, 
  ExtractFilePath(Path),  False -  }
{ if Path is a directory name then works in the same way as GetFileList else
  calls GetFileList(List, ExtractFilePath(Path), ExtractFileName(Path), SubDirs);
  returns True if either Path or ExtractFilePath(Path) directory exist and
  False otherwise }

function GetDirList(List: TStrLst; const Path, Mask: String): Integer;
{   Path    List   Path (
   ),   Mask;   
   }
{ scans the directory Path and writes names of subdirectories of Path matching
  Mask to the List (without full paths); returns the number of the subdirectories
  found }

function GetModuleName(Module: HMODULE): String;
{ returns the name of the file containing the specified module }

function FileTimeToDateTime(const Date: TFileTime): TDateTime;
function DateTimeToFileTime(const Date: TDateTime): TFileTime;

implementation

function LockFileWrite(const FileName: String): THandle;
begin
  Result:=CreateFile(PChar(FileName), GENERIC_READ, FILE_SHARE_READ, nil,
    OPEN_EXISTING, FILE_FLAG_NO_BUFFERING, 0);
end;

function UnlockFileWrite(const Handle: THandle): Boolean;
begin
  Result:=CloseHandle(Handle);
end;

function GetLongFileName(const Name: String): String;

  function ProcessExpanded(const ExpName: String): String;
  var
    I: Integer;
    S: String;
    SR: TSearchRec;
  begin
    Result:=ExpName;
    I:=Length(Result);
    if I > 0 then begin
      if Result[I] = '\' then begin
        Dec(I);
        if (I > 0) and (Result[I] = ':') then Exit;
        SetLength(Result, I);
      end
      else
        if Result[I] = ':' then Exit;
      S:=ExtractFileName(Result);
      if (Pos('*', S) = 0) and (Pos('?', S) = 0) then begin
        if SysUtils.FindFirst(Result, faAnyFile and not faVolumeID, SR) = 0 then
          SysUtils.FindClose(SR)
        else
          Exit;
        if SR.Name = '.' then Exit; // ...
      end
      else
        SR.Name:=S;
      S:=ExtractFilePath(Result);
      if Length(S) < I then begin
        S:=ProcessExpanded(S);
        I:=Length(S);
        if (I > 0) and (S[I] <> '\') then S:=S + '\';
        Result:=S + SR.Name;
      end;
    end;
  end;

begin
  Result:=ExpandFileName(Name);
  if Pos('~', Result) > 0 then Result:=ProcessExpanded(Result);
end;

function DirectoryExists(const Name: String): Boolean;
var
  Code: DWORD;
begin
  Code:=GetFileAttributes(PChar(Name));
  Result:=(Code <> DWORD(-1)) and (Code and FILE_ATTRIBUTE_DIRECTORY <> 0);
end;

function ForceDirectories(Dir: String): Boolean;
var
  L: Integer;
begin
  Result:=True;
  L:=Length(Dir);
  if L = 0 then
    raise Exception.Create('Cannot create directory');
  if Dir[L] = '\' then SetLength(Dir, L - 1);
  if (Length(Dir) < 3) or DirectoryExists(Dir)
    or (ExtractFilePath(Dir) = Dir) then Exit; // avoid 'xyz:\' problem.
  Result:=ForceDirectories(ExtractFilePath(Dir)) and CreateDir(Dir);
end;

function ForceDeleteFileOrDir(const Name: String): Boolean;
var
  Attr: DWORD;
begin
  Result:=False;
  Attr:=GetFileAttributes(PChar(Name));
  if Attr <> DWORD($FFFFFFFF) then begin
    if Attr and FILE_ATTRIBUTE_READONLY <> 0 then
      SetFileAttributes(PChar(Name), Attr and not FILE_ATTRIBUTE_READONLY);
    if Attr and FILE_ATTRIBUTE_DIRECTORY = 0 then
      Result:=DeleteFile(Name)
    else
      Result:=RemoveDirectory(PChar(Name));
  end;
end;

procedure ForceDeleteFiles(List: TStrLst);
var
  I: Integer;
begin
  for I:=0 to List.Count - 1 do
    ForceDeleteFileOrDir(List[I]);
end;

function ExcludeFileExt(const Name: String): String;
begin
  Result:=Name;
  SetLength(Result, Length(Result) - Length(ExtractFileExt(Name)));
end;

function GetFileList(List: TStrLst; const Path, Mask: String; SubDirs: Boolean): Integer;

  procedure AddToList(CurPath: String);
  var
    I, Code: Integer;
    SR: TSearchRec;
    SubDirList: TStrLst;
  begin
    if CurPath = '' then Exit;
    if CurPath[Length(CurPath)] <> '\' then CurPath:=CurPath + '\';
    Code:=FindFirst(CurPath + Mask,
      faReadOnly + faHidden + faSysFile + faArchive, SR);
    if Code = 0 then
    try
      repeat
        List.Add(CurPath + SR.Name);
        Code:=FindNext(SR);
      until Code <> 0;
    finally
      SysUtils.FindClose(SR);
    end;
    if SubDirs then begin
      SubDirList:=TStrLst.Create;
      try
        Code:=FindFirst(CurPath + '*',
          faReadOnly + faHidden + faSysFile + faArchive + faDirectory, SR);
        if Code = 0 then
        try
          repeat
            if (SR.Attr and faDirectory	<> 0) { bug workaround } and
              (SR.Name <> '.') and (SR.Name <> '..')
            then
              SubDirList.Add(SR.Name);
            Code:=FindNext(SR);
          until Code <> 0;
        finally
          SysUtils.FindClose(SR);
        end;
        for I:=0 to SubDirList.Count - 1 do
          AddToList(CurPath + SubDirList.Items[I]);
      finally
        SubDirList.Free;
      end;
    end;
  end;

begin
  List.Clear;
  AddToList(Path);
  Result:=List.Count;
end;

function GetFileListEx(List: TStrLst; Path, Mask: String; SubDirs: Boolean): Boolean;
var
  L: Integer;
begin
  if not DirectoryExists(Path) then begin
    Result:=False;
    Mask:=ExtractFileName(Path);
    L:=Length(Path) - Length(Mask) - 1;
    if L <= 0 then Exit;
    SetLength(Path, L);
    if not DirectoryExists(Path) then Exit;
  end;
  GetFileList(List, Path, Mask, SubDirs);
  Result:=True;
end;

function GetDirList(List: TStrLst; const Path, Mask: String): Integer;
var
  Code: Integer;
  SR: TSearchRec;
begin
  Code:=FindFirst(Path + '*', faReadOnly + faHidden + faArchive + faDirectory, SR);
  if Code = 0 then
  try
    repeat
      if (SR.Attr and faDirectory <> 0) { bug workaround } and
        (SR.Name <> '.') and (SR.Name <> '..')
      then
        List.Add(SR.Name);
      Code:=FindNext(SR);
    until Code <> 0;
  finally
    SysUtils.FindClose(SR);
  end;
  Result:=List.Count;
end;

// from SysUtils implementation
function GetModuleName(Module: HMODULE): String;
var
  ModName: array [0..MAX_PATH] of Char;
begin
  SetString(Result, ModName, Windows.GetModuleFileName(Module, ModName, SizeOf(ModName)));
end;

function FileTimeToDateTime(const Date: TFileTime): TDateTime;
var
  ST: TSystemTime;
begin
  FileTimeToSystemTime(Date, ST);
  Result:=SystemTimeToDateTime(ST);
end;

function DateTimeToFileTime(const Date: TDateTime): TFileTime;
var
  ST: TSystemTime;
begin
  DateTimeToSystemTime(Date, ST);
  SystemTimeToFileTime(ST, Result);
end;

end.
