{ Version 010927. Copyright  Alexey A.Chernobaev, 1996-2001 }

{ Some functions of this unit are based on RX Library (unit StrUtils)
  Copyright (c) 1995, 1996 AO ROSNO; Copyright (c) 1997, 1998 Master-Bank;
  StrUtils is based on AlexGraf String Library by Alexei Lukin (c) 1992 }

unit VFormat;

interface

{$I VCheck.inc}

uses
  ExtType, SysUtils, VectStr, VectErr;

type
  TFormat = (WindowsLong, WindowsShort, RussianLong, RussianShort,
    EnglishLong, EnglishShort, Simple);
{
         :
   Windows-,  Windows-,
   '<> <  -> <>',
   dd.mm.yyyy,
   '<  -> <>,  <>',
   mm/dd/yyyy,  yyyymmdd.

   : Windows-  , Windows-  ,
   hh:mm:ss,  hh:mm,  'hh:mm:ss AM|PM',  hh <= 12,
   'hh:mm AM|PM',  hh <= 12,  hhmm.

    : Yes/No, Y/N, /, /, True/False, T/F, 1/0.

  For dates constants of this enumarated type stand for:
  long Windows-format, short Windows-format,
  format '<day> <russian name of month> <year>',
  format dd.mm.yyyy,
  format '<english name of month> <day>,  <year>',
  format mm/dd/yyyy, format yyyymmdd.

  For time: Windows-format with seconds, Windows-format without seconds,
  format hh:mm:ss, format hh:mm, format 'hh:mm:ss AM|PM', where hh <= 12,
  format 'hh:mm AM|PM', where hh <= 12, format hhmm.

  For boolean values: Yes/No, Y/N, /, /, True/False, T/F, 1/0.
}

const
  RealFormat: String = '%g'; {      }
  DateFormat: TFormat = RussianShort;
  TimeFormat: TFormat = RussianShort;
  BoolFormat: TFormat = WindowsShort;

function StdStrToInt(const Value: String): Integer;
function StdStrToInt64(const Value: String): Int64;
{     ; Value     
      (    
    '$'  '0x'    'h') }
{ converts string to integer; Value can be in decimal or hexadecimal formats
  (in latter case the string either begins from '$' or '0x' or ends with 'h') }

function StringToReal(Value: String): Extended;
{     ;     
     '.', ','    DecimalSeparator }
{ converts string to floating-point number; it's possible to use '.', ',' and
  system-defined DecimalSeparator for delimiting integral and fractional parts }

function RealToString(Value: Extended): String;
{     ,  RealFormat;  
        '.' }
{ converts floating-point number to string using RealFormat and '.' as the
  decimal separator }

function RealToStringF(Value: Extended; Precision, Digits: Integer): String;
{     ,  ffGeneral  
  ;         '.' }
{ converts floating-point number to string using ffGeneral and '.' as the
  decimal separator }

{$IFNDEF NO_DATETIME}
function StdDateToStr(ADate: TDateTime): String;
{    ,   DateFormat;    ,
      }
{ converts date to string using DateFormat; if year isn't specified then current
  year is used }

function StdStrToDate(const Value: String): TDateTime;
{  ,      TFormat,   }
{ converts string which can be in every of formats TFormat to date }

function StdTimeToStr(ATime: TDateTime): String;
{    ,   TimeFormat }
{ converts time to string using TimeFormat format }

function StdStrToTime(Value: String): TDateTime;
{  ,      TTimeFormat,   }
{ converts string which can be in every of formats TTimeFormat to time }

function StdDateTimeToStr(ADate: TDateTime): String;
{  -  ,   DateFormat  TimeFormat }
{ converts date-time to string using formats DateFormat and TimeFormat }

function StdStrToDateTime(Value: String): TDateTime;
{  ,      TFormat,  - }
{ converts string which can be in every of formats TFormat to date-time }
{$ENDIF}

function CharToStr(C: Char): String;
{    ;  C < #32,  Result:='#' + IntToStr(Ord(C)) }
{ converts character to string; if C < #32 then Result:='#' + IntToStr(Ord(C)) }

function StrToChar(const Value: String): Char;
{ ,   CharToStr }
{ function inverse to CharToStr }

function BoolToStr(B: Boolean): String;
{      }
{ converts boolean to string }

function StrToBool(Value: String): Boolean;
{ ,   BoolToStr }
{ function inverse to BoolToStr }

function StandardFormat(const FormatStr: String; const Args: array of const): String;
{   Format,    '.'   
       }
{ analog of Format function which always uses '.' as the decimal separator }

implementation

function PrepareInt(const Value: String): String;
begin
  Result:=Trim(Value);
  if Result <> '' then begin
    if UpCase(Result[Length(Result)]) = 'H' then
      Result:='$' + Copy(Result, 1, Length(Result) - 1)
    else if Pos('0X', UpperCase(Result)) = 1 then
      Result:='$' + Copy(Result, 3, Length(Result));
  end;
end;

function StdStrToInt(const Value: String): Integer;
begin
  try
    Result:=StrToInt(PrepareInt(Value));
  except
    raise Exception.CreateFmt(SCanNotConvertToInteger, [Value]);
  end;
end;

{$IFDEF V_D4}
function StdStrToInt64(const Value: String): Int64;
begin
  try
    Result:=StrToInt64(PrepareInt(Value));
  except
    raise Exception.CreateFmt(SCanNotConvertToInteger, [Value]);
  end;
end;
{$ELSE}
function StdStrToInt64(const Value: String): Int64;
var
  I, L: Integer;
  S: String;
  Hex: Boolean;
  F: Float64;
begin
  S:=Trim(Value);
  L:=Length(S);
  try
    if L = 0 then Abort;
    Hex:=False;
    if UpCase(S[L]) = 'H' then begin
      Dec(L);
      SetLength(S, L);
      Hex:=True;
    end
    else if S[1] = '$' then begin
      Dec(L);
      Delete(S, 1, 1);
      Hex:=True;
    end;
    if Hex then begin
      if L = 0 then Abort;
      I:=L - 7;
      if I < 1 then I:=1;
      QWordRec(Result).Lo:=StrToInt('$' + Copy(S, I, 8));
      S:=Copy(S, 1, I - 1);
      if S <> '' then
        QWordRec(Result).Hi:=StrToInt('$' + S)
      else
        QWordRec(Result).Hi:=0;
    end
    else begin
      F:=StringToReal(S);
      if Frac(F) <> 0 then Abort;
      Result:={$IFNDEF INT64_EQ_COMP}Round{$ENDIF}(F);
    end;
  except
    raise Exception.CreateFmt(SCanNotConvertToInteger, [Value]);
  end;
end;
{$ENDIF}

{$IFDEF NOWARN}{$WARNINGS OFF}{$ENDIF}
function StringToReal(Value: String): Extended;
var
{$IFDEF V_DELPHI}
  OldSeparator: Char;
{$ELSE}
  I: Integer;
{$ENDIF}
begin
  Value:=Trim(Value);
  {$IFDEF V_DELPHI}
  OldSeparator:=DecimalSeparator;
  try
    if Pos('.', Value) <> 0 then DecimalSeparator:='.'
    else if Pos(',', Value) <> 0 then DecimalSeparator:=',';
    try
      Result:=StrToFloat(Value);
    except
      raise Exception.CreateFmt(SCanNotConvertToFloat, [Value]);
    end;
  finally
    DecimalSeparator:=OldSeparator;
  end;
  {$ELSE}
  I:=Pos(',', Value);
  if I > 0 then Value[I]:='.';
  Val(Value, Result, I);
  if I <> 0 then raise Exception.CreateFmt(SCanNotConvertToFloat, [Value]);
  {$ENDIF}
end;
{$IFDEF NOWARN}{$WARNINGS ON}{$ENDIF}

function RealToString(Value: Extended): String;
var
  OldDecimalSeparator: Char;
begin
  OldDecimalSeparator:=DecimalSeparator;
  try
    DecimalSeparator:='.';
    Result:=Format(RealFormat, [Value]);
  finally
    DecimalSeparator:=OldDecimalSeparator;
  end;
end;

function RealToStringF(Value: Extended; Precision, Digits: Integer): String;
var
  OldDecimalSeparator: Char;
begin
  OldDecimalSeparator:=DecimalSeparator;
  try
    DecimalSeparator:='.';
    Result:=FloatToStrF(Value, ffGeneral, Precision, Digits);
  finally
    DecimalSeparator:=OldDecimalSeparator;
  end;
end;

{$IFNDEF NO_DATETIME}
function WToS2(W: Word): String;
begin
  Result:=AddChar('0', IntToStr(W), 2);
end;

type
  TMonth = 1..12;

function StdDateToStr(ADate: TDateTime): String;
const
  MonthNamesRus: array [TMonth] of String[11] = ('', '', '',
    '', '', '', '', '', '', '', '',
    '');
  MonthNamesEng: array [TMonth] of String[9] = ('January', 'February', 'March',
    'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November',
    'December');
var
  Year, Month, Day: Word;
  S: String;
begin
  Case DateFormat of
    WindowsLong: begin
      S:=ShortDateFormat;
      try
        ShortDateFormat:=LongDateFormat;
        Result:=DateToStr(ADate);
      finally
        ShortDateFormat:=S;
      end;
    end;
    WindowsShort:
      Result:=DateToStr(ADate);
    RussianLong: begin
      DecodeDate(ADate, Year, Month, Day);
      Result:=IntToStr(Day) + ' ' + MonthNamesRus[Month] + ' ' + IntToStr(Year) + ' .';
    end;
    RussianShort: begin
      DecodeDate(ADate, Year, Month, Day);
      Result:=WToS2(Day) + '.' + WToS2(Month) + '.' + IntToStr(Year);
    end;
    EnglishLong: begin
      DecodeDate(ADate, Year, Month, Day);
      Result:=MonthNamesEng[Month] + ' ' + IntToStr(Day) + ', ' + IntToStr(Year);
    end;
    EnglishShort: begin
      DecodeDate(ADate, Year, Month, Day);
      Result:=WToS2(Month) + '/' + WToS2(Day) + '/' + IntToStr(Year);
    end;
  Else begin {Simple}
    DecodeDate(ADate, Year, Month, Day);
    Result:=AddChar('0', IntToStr(Year), 4) + WToS2(Month) + WToS2(Day);
  end;
  End;
end;

function FStdStrToDate(const Value: String; var LastPos: Integer): TDateTime;
const
  { ,      }
  MonthRus: array [TMonth] of string[3] = ('', '', '',
    '', '', '', '', '', '', '', '', '');
  MonthEng: array [TMonth] of string[3] = ('JA', 'F', 'MAR',
    'AP', 'MAY', 'JUN', 'JUL', 'AU', 'S', 'O', 'N', 'D');
  Delimiters = [' ', '.', ',', ';', '/', '\', '-', '_'];
type
  TCharSet = set of Char;
var
  LPos: Integer;
  S: String;

  function PassChars(const Chars: TCharSet; Equal: Boolean): Boolean;
  begin
    Result:=False;
    if LPos <= Length(S) then begin
      while (S[LPos] in Chars) = Equal do begin
        Inc(LPos);
        if LPos > Length(S) then Exit;
      end;
      Result:=True;
    end;
  end;

  function GetMonth(const S: String): Integer;
  var
    I: Integer;
  begin
    for I:=1 to 12 do
      if (Pos(MonthRus[I], S) = 1) and ((I <> 5) or (Length(S) > 2) and
        (S[3] in ['', ''])) or (Pos(MonthEng[I], S) = 1)
      then begin
        Result:=I;
        Exit;
      end;
    Result:=0;
  end;

var
  Year, Month, Day, MM, DD: Word;
  I, J, K, L: Integer;
  DelimStart, DelimEnd: Integer;
  HasSuffix: Boolean;
  T: String;
begin
  Year:=0;
  S:=AnsiUpperCase(Trim(Value));
  Month:=GetMonth(S);
  LPos:=1;
  try
    if Month > 0 then begin
      if not PassChars(Digits, False) then Abort;
      I:=LPos;
      J:=I;
      if PassChars(Digits, True) then begin
        I:=LPos;
        if PassChars(Digits, False) then begin
          K:=LPos;
          PassChars(Digits, True);
          L:=LPos;
          if L <> K then Year:=StrToInt(Copy(S, K, L - K));
        end;
      end;
      Day:=StrToInt(Copy(S, J, I - J));
    end
    else begin
      if not PassChars(Digits, True) then Abort;
      I:=LPos;
      if I = 1 then Abort;
      Day:=StrToInt(Copy(S, 1, I - 1));
      DelimStart:=I;
      PassChars(Delimiters, True);
      I:=LPos;
      DelimEnd:=I;
      Month:=GetMonth(Copy(S, I, Length(S)));
      if Month = 0 then begin
        J:=I;
        HasSuffix:=PassChars(Digits, True);
        I:=LPos;
        Month:=StrToInt(Copy(S, J, I - J));
        T:=Trim(Copy(S, DelimStart, DelimEnd - DelimStart));
        if (T <> '') and ((T[1] = '/') or
          (T[1] <> '.') and (AnsiUpperCase(ShortDateFormat)[1] in ['M', '']))
        then begin
          L:=Day;
          Day:=Month;
          Month:=L;
        end;
      end
      else
        HasSuffix:=True;
      if HasSuffix and PassChars(Digits, False) then begin
        K:=LPos;
        PassChars(Digits, True);
        L:=LPos;
        Year:=StrToInt(Copy(S, K, L - K));
      end;
    end;
    PassChars(Digits, False);
    if Year = 0 then DecodeDate(Date, Year, MM, DD);
    Result:=EncodeDate(Year, Month, Day);
    LastPos:=LPos;
  except
    raise Exception.CreateFmt(SIllegalDateTime, [Value]);
  end;
end;

function StdStrToDate(const Value: String): TDateTime;
var
  LastPos: Integer;
begin
  Result:=FStdStrToDate(Value, LastPos);
end;

function StdTimeToStr(ATime: TDateTime): String;

  function EnglishTime(English, Short: Boolean): String;
  var
    HH, MM, SS, MS: Word;
    S: String;
  begin
    DecodeTime(ATime, HH, MM, SS, MS);
    if English then
      if HH > 12 then begin
        Dec(HH, 12);
        S:='PM';
      end
      else
        S:='AM'
    else
      S:='';
    Result:=WToS2(HH) + ':' + WToS2(MM);
    if not Short then Result:=Result + ':' + WToS2(SS);
    if S <> '' then Result:=Result + ' ' + S;
  end;

var
  HH, MM, SS, MS: Word;
  S: String;
begin
  Case TimeFormat of
    WindowsLong:
      Result:=TimeToStr(ATime);
    WindowsShort: begin
      S:=LongTimeFormat;
      try
        LongTimeFormat:=ShortTimeFormat;
        Result:=TimeToStr(ATime);
      finally
        LongTimeFormat:=S;
      end;
    end;
    RussianLong:
      Result:=EnglishTime(False, False);
    RussianShort:
      Result:=EnglishTime(False, True);
    EnglishLong:
      Result:=EnglishTime(True, False);
    EnglishShort:
      Result:=EnglishTime(True, True);
  Else begin {Simple}
    DecodeTime(ATime, HH, MM, SS, MS);
    Result:=WToS2(HH) + WToS2(MM);
  end;
  End;
end;

function StdStrToTime(Value: String): TDateTime;
begin
  Value:=Trim(Value);
  if TimeSeparator <> ':' then Value:=ReplaceStr(Value, ':', TimeSeparator);
  if Pos(TimeSeparator, Value) <= 0 then
    raise Exception.CreateFmt(SIllegalDateTime, [Value]);
  Result:=StrToTime(Value);
end;

function StdDateTimeToStr(ADate: TDateTime): String;
begin
  Result:=StdDateToStr(ADate) + ' ' + StdTimeToStr(ADate);
end;

function StdStrToDateTime(Value: String): TDateTime;
var
  LastChar: Integer;
  Negative: Boolean;
begin
  Value:=Trim(Value);
  Result:=FStdStrToDate(Value, LastChar);
  if Value[LastChar - 1] in [':', TimeSeparator] then
    raise Exception.CreateFmt(SIllegalDateTime, [Value]);
  Value:=Trim(Copy(Value, LastChar, Length(Value)));
  if Value <> '' then begin
    Negative:=Result < 0;
    Result:=Abs(Result) + StdStrToTime(Value);
    if Negative then Result:=-Result;
  end;
end;
{$ENDIF}

function CharToStr(C: Char): String;
begin
  if C >= #32 then Result:=C else Result:='#' + IntToStr(Ord(C));
end;

function StrToChar(const Value: String): Char;
var
  I, Code: Integer;
begin
  if Length(Value) = 1 then Result:=Value[1]
  else if (Length(Value) > 1) and (Value[1] = '#') then begin
    Val(Copy(Value, 2, Length(Value)), I, Code);
    if (Code <> 0) or not (I in [0..255]) then
      raise Exception.CreateFmt(SIllegalChar, [Value]);
    Result:=Chr(I);
  end
  else
    raise Exception.CreateFmt(SIllegalChar, [Value]);
end;

function BoolToStr(B: Boolean): String;
begin
  Case BoolFormat of
    WindowsLong: if B then Result:='Yes' else Result:='No';
    WindowsShort: if B then Result:='Y' else Result:='N';
    RussianLong: if B then Result:='' else Result:='';
    RussianShort: if B then Result:='' else Result:='';
    EnglishLong: if B then Result:='True' else Result:='False';
    EnglishShort: if B then Result:='T' else Result:='F';
  Else {Simple}
    if B then Result:='1' else Result:='0';
  End;
end;

function StrToBool(Value: String): Boolean;
begin
  Value:=AnsiUpperCase(Trim(Value));
  if (Value = '1') or (Value = 'YES') or (Value = 'Y') or (Value = '') or
    (Value = '') or (Value = 'TRUE') or (Value = 'T')
  then
    Result:=True
  else if (Value = '0') or (Value = 'NO') or (Value = 'N') or (Value = '') or
    (Value = '') or (Value = 'FALSE') or (Value = 'F')
  then
    Result:=False
  else
    raise Exception.CreateFmt(SIllegalBool, [Value]);
end;

function StandardFormat(const FormatStr: String; const Args: array of const): String;
var
  OldDecimalSeparator: Char;
begin
  OldDecimalSeparator:=DecimalSeparator;
  try
    DecimalSeparator:='.';
    Result:=Format(FormatStr, Args);
  finally
    DecimalSeparator:=OldDecimalSeparator;
  end;
end;

end.
