{ Version 010927. Copyright  Alexey A.Chernobaev, 1996-2001 }

{ Some functions of this unit are based on RX Library (unit StrUtils)
  Copyright (c) 1995, 1996 AO ROSNO; Copyright (c) 1997, 1998 Master-Bank }

unit VectStr;

interface

{$I VCheck.inc}

uses
  ExtSys, SysUtils
  {$IFDEF V_WIN},{$IFNDEF WIN32}WinTypes, WinProcs{$ELSE}Windows{$ENDIF}{$ENDIF};

const
  Digits = ['0'..'9'];
  Alpha =  ['A'..'Z', 'a'..'z'];
  AlphaNumeric = Alpha + Digits;

type
  TCharSet = set of Char;

function NumOfChars(C: Char; const S: String): Integer;
{$IFDEF V_32}
function NumOfWideChars(C: WideChar; const S: WideString): Integer;
{$ENDIF}
{     C   S }
{ returns the number of occurrences of the character C in the string S }

function NumOfSubStr(const SubS, S: String): Integer;
{$IFDEF V_32}
function NumOfSubStrWide(const SubS, S: WideString): Integer;
{$ENDIF}
{     SubS   S }
{ returns the number of occurrences of the substring SubS in the string S }

function CharPos(C: Char; const S: String; From: Integer): Integer;
{         S,   
  From (>= 1);  0,     }
{ returns the index of the first occurrence of the character C in the string S
  starting from position From (>= 1); returns 0 if the character not found }

function LastPos(C: Char; const S: String): Integer;
{   C    S,     
   ,   0 }
{ if the string S contains the character C then returns the index of the last
  occurrence of this character, otherwise returns 0 }

function FirstUpper(const S: String): String;
{    S    }
{ changes the first character of S to uppercase }

function FirstLower(const S: String): String;
{    S    }
{ changes the first character of S to lowercase }

function DelLastIf(const S: String; C: Char): String;
{   S   ,      1 
   ,    S }
{ if the string S terminates with the character C then decreases the string
  length by 1 and returns it, otherwise returns a copy of S }

function EnsureLast(const S: String; C: Char): String;
{   S    ,   (S + C), 
    S }
{ if the string S doesn't terminate with the character C then returns (S + C),
  otherwise returns a copy of S }

function PosFrom(const SubStr, S: String; From: Integer): Integer;
{  System.Pos,  SubStr ,    From (>=1) }
{ analog of System.Pos which searches for SubStr from the position From (>=1) }
{$IFDEF V_32}
function PosFromWide(const SubStr, S: WideString; From: Integer): Integer;
{$ENDIF}

function StartsWith(const S, What: String): Boolean;
{ Result = (Copy(S, 1, Length(What)) = What) }

{$IFNDEF V_32}
procedure SetLength(var S: String; ALength: Byte);

function Trim(const S: String): String;
{     <= ' '      }
{ removes leading and trailing characters with codes <= ' ' from the string }
{$ENDIF}

function MakeStr(C: Char; N: Integer): String;
{  RX Library:  ,   N  C }
{ from RX Library: returns the string consisting of N characters C }

function AddChar(C: Char; const S: String; N: Integer): String;
{  RX Library:   S,    N  C  }
{ from RX Library: returns copy of S left-padded to length N with characters C }

function ReplaceStr(const Value, FromStr, ToStr: String): String;
{  RX Library:  ,   Value   
  FromStr  ToStr }
{ from RX Library: returns the result of replacing all occurrences of FromStr in
  Value to ToStr }

procedure ReplaceCharV(var S: String; FromChar, ToChar: Char);
function ReplaceChar(const S: String; FromChar, ToChar: Char): String;
{  ,   Value    FromChar 
  ToChar }
{ returns the result of replacing all occurrences of the FromChar character in
  Value to ToChar }

function ContainsChars(const S: String; Chars: TCharSet): Boolean;
{ ,   S    Chars }
{ checks whether S contains one of characters Chars or not }

function DelDupChar(const S: String; C: Char): String;
{  ,   S    C (
  ,       C   ) }
{ returns the result of deleting duplicate characters C in the string S (in other
  words, only one character remains from every group of adjacent characters C) }

{$IFDEF V_32}
function DecodeCEscapes(const S: String): String;
{  ,   S   escape-
     ASCII-,    
   printf      C (,
  "\n"   #13#10, "\t" -  #9  ..) }
{ returns the result of replacing all escape-sequences in the string S to the
  corresponding sequences of ASCII codes as the printf-family functions of the
  standard C-language run-time library do (e.g. "\n" will be replaced to #13#10,
  "\t" - to #9, etc.) }

function EncodeCEscapes(const S: String; Only7Bit: Boolean): String;
{ ,   DecodeCEscapes;  Only7Bit = True,    
   >= #128   escape- \xNN,  NN -
      }
{ inverse function for DecodeCEscapes; if Only7Bit = True then all characters
  with codes >= #128 will be encoded with \xNN escape-sequences, where NN is the
  hexadecimal representation of the character code }
{$ENDIF}

{$IFDEF WIN32}
function WordPos(SubWord, S: String; CaseSensitive: Boolean): Integer;
{  SubWord    S   ,    
   ,   0 }
{ if the string S contains SubWord as the whole word then returns the index of
  the first such occurrence, otherwise returns 0 }

function AnsiToOEM(const S: String): String;
function OEMToAnsi(const S: String): String;
{$ENDIF}

function IsCorrectIdentifier(const S: String): Boolean;
{ ,    S   (.. 
        ,    , 
      ) }
{ checks whether the string S is the correct identifier (i.e. the string is not
  empty and contains only latin characters, digits and '_' characters, moreover,
  first character isn't a digit) }

function RemoveComment(const S: String; CommentPrefix: Char): String;
{   S  -  ,    CommentPrefix,
    ,   CommentPrefix    
  ,     ;  
    ,        
    (   <= ' '). :
  RemoveComment('Point="2.0;3.5" ; coordinates', ';') = 'Point="2.0;3.5"' }
{ removes any comments from the string S; comment is the part of the string
  starting with CommentPrefix character, including this symbol, if only
  CommentPrefix isn't located inside the string literal enclosed in the single or
  double quotes; returns the remaining part of the string with deleted leading
  and trailing spaces and special characters (characters with codes <= ' ').
  E.g.: RemoveComment('Point="2.0;3.5" ; coordinates', ';') = 'Point="2.0;3.5"' }

function StringToLiteral(const S: String): String;
{$IFDEF V_32}
function WideStringToLiteral(const S: WideString): WideString;
{$ENDIF}
{   S  ,  S     
   ,   S. : 'cause => '''cause' }
{ converts the string S to the literal enclosing S in the single quotes and
  duplicating single quotes inside S. E.g.: 'cause => '''cause' }

function StringToLiteral2(const S: String): String;
{$IFDEF V_32}
function WideStringToLiteral2(const S: WideString): WideString;
{$ENDIF}
{   S  ,  S     
   ,   S. : "cause => """cause" }
{ converts the string S to the literal enclosing S in the double quotes and
  duplicating double quotes inside S. E.g.: "cause => """cause" }

function TextToLiteral(const S: String): String;
{$IFDEF V_32}
function WideTextToLiteral(const S: WideString): WideString;
{$ENDIF}
{    S  ,    ,    
   (. CheckText),  S -  ,   S  ,
   S       ,   S,
     S }
{ if there are characters differing from the Latin letters, digits and some
  other characters (see CheckText) in the string S or S is the empty string then
  converts S to the literal enclosing S in the single quotes and duplicating
  single quotes inside S, otherwise returns the copy of S }

function TextToLiteral2(const S: String): String;
{    S  ,    ,    
   (. CheckText),  S -  ,   S  ,
   S       ,   S,
     S }
{ if there are characters differing from the Latin letters, digits and some
  other characters (see CheckText) in the string S or S is the empty string then
  converts S to the literal enclosing S in the double quotes and duplicating
  double quotes inside S, otherwise returns the copy of S }

function LiteralToString(const S: String): String;
{$IFDEF V_32}
function LiteralToWideString(const S: WideString): WideString;
{$ENDIF}
{    StringToLiteral:       
  S,           ;
        (  
       ,   
  ,     ). : '''cause' => 'cause;
  "'cause" => 'cause; """London""" => "London"; "'London'" => 'London';  S
      (    ),    S }
{ inverse function for StringToLiteral; accepts both single and double quotes.
  E.g.: '''cause' => 'cause; "'cause" => 'cause; """London""" => "London";
  "'London'" => 'London'; if the string S isn't enclosed in any quotes (the same
  at the beginning and at the end of the string) then returns the copy of S }

const
{$IFDEF WIN32}
  iSystemLocale = LOCALE_SYSTEM_DEFAULT; { $0800: Use system default }
  iUserLocale = LOCALE_USER_DEFAULT; { $0400: Use user default}

  iCaseInsensitive = NORM_IGNORECASE; { $0001: Case-insensitive index }
  iIgnoreNonSpace = NORM_IGNORENONSPACE; { $0002: Ignore non-spacing characters }
  iIgnoreSymbols = NORM_IGNORESYMBOLS; { $0004: Ignore symbols }
  iStringSort = SORT_STRINGSORT; { $1000: Use string-order instead of word-order }
  FlagsMask = iCaseInsensitive or iIgnoreNonSpace or iIgnoreSymbols or iStringSort;
{$ELSE}
  iSystemLocale = $0800; { iSystemLocale and iUserLocale are the same for D1 }
  iUserLocale = $0400;

  iCaseInsensitive = $0001; { Case-insensitive index }
  FlagsMask = iCaseInsensitive;
{$ENDIF}
  LocaleMask = iSystemLocale or iUserLocale;

  { iLocaled: use user locale, ascending, case insensitive, word sort order }
  iLocaled = iUserLocale;
  { iDefault: don't use locale, use ascending, case insensitive, word sort order }
  iDefault = 0;

function CmpStrF(const S1, S2: String; Flags: LongInt): Integer;
{ Result value >0 if S1 greater then S2, <0 if S1 less then S2, and 0 if equal.
  If Flags = 0 then CmpStrF compares S1 and S2 in the same way as if they were
  compared using "<" and ">" operators.
  If Flags <> 0 then CmpStrF uses Windows API CompareString function which
  supports several methods for comparing strings: at first, it can use either
  System Locale or Current User Locale; then, it can treat strings in the
  case-sensitive or case-insensitive manner; and, at last, it uses "word sort"
  or "string sort" variants (see Windows Help for details). Specify the mode you
  want in the Flags parameter (see constants above). }

implementation

function NumOfChars(C: Char; const S: String): Integer;
var
  I: Integer;
begin
  Result:=0;
  for I:=1 to Length(S) do if S[I] = C then Inc(Result);
end;

{$IFDEF V_32}
function NumOfWideChars(C: WideChar; const S: WideString): Integer;
var
  I: Integer;
begin
  Result:=0;
  for I:=1 to Length(S) do if S[I] = C then Inc(Result);
end;
{$ENDIF}

function NumOfSubStr(const SubS, S: String): Integer;
var
  I: Integer;
begin
  Result:=0;
  I:=0;
  repeat
    I:=PosFrom(SubS, S, I);
    if I = 0 then Exit;
    Inc(Result);
    Inc(I, Length(SubS));
  until False;
end;

{$IFDEF V_32}
function NumOfSubStrWide(const SubS, S: WideString): Integer;
var
  I: Integer;
begin
  Result:=0;
  I:=0;
  repeat
    I:=PosFromWide(SubS, S, I);
    if I = 0 then Exit;
    Inc(Result);
    Inc(I, Length(SubS));
  until False;
end;
{$ENDIF}

function CharPos(C: Char; const S: String; From: Integer): Integer;
var
  I, J: Integer;
begin
  {$IFDEF V_LONGSTRINGS}
  I:=Length(S);
  if From <= I then begin
    if From < 1 then From:=1;
    J:=From - 1;
    Result:=IndexOfValue8((PChar(S) + J)^, Ord(C), I - J);
    if Result >= 0 then Inc(Result, From);
  end
  else
    Result:=0;
  {$ELSE}
  for I:=From to Length(S) do
    if S[I] = C then begin
      Result:=I;
      Exit;
    end;
  Result:=0;
  {$ENDIF}
end;

function LastPos(C: Char; const S: String): Integer;
var
  I: Integer;
begin
  Result:=0;
  for I:=Length(S) downto 1 do
    if S[I] = C then begin
      Result:=I;
      Exit;
    end;
end;

function FirstUpper(const S: String): String;
var
  Buf: array [0..1] of Char;
begin
  Result:=S;
  if S <> '' then begin
    Buf[0]:=S[1];
    Buf[1]:=#0;
    AnsiUpper(Buf);
    Result[1]:=Buf[0];
  end;
end;

function FirstLower(const S: String): String;
var
  Buf: array [0..1] of Char;
begin
  Result:=S;
  if S <> '' then begin
    Buf[0]:=S[1];
    Buf[1]:=#0;
    AnsiLower(Buf);
    Result[1]:=Buf[0];
  end;
end;

function DelLastIf(const S: String; C: Char): String;
begin
  Result:=S;
  if (S <> '') and (S[Length(S)] = C) then SetLength(Result, Length(S) - 1);
end;

function EnsureLast(const S: String; C: Char): String;
begin
  Result:=S;
  if (S <> '') and (S[Length(S)] <> C) then Result:=Result + C;
end;

function PosFrom(const SubStr, S: String; From: Integer): Integer;
begin
  if From < 1 then From:=1;
  Result:=Pos(SubStr, Copy(S, From, Length(S)));
  if Result > 0 then Inc(Result, From - 1);
end;

{$IFDEF V_32}
function PosFromWide(const SubStr, S: WideString; From: Integer): Integer;
begin
  if From < 1 then From:=1;
  Result:=Pos(SubStr, Copy(S, From, Length(S)));
  if Result > 0 then Inc(Result, From - 1);
end;
{$ENDIF}

function StartsWith(const S, What: String): Boolean;
var
  I, L: Integer;
begin
  Result:=False;
  L:=Length(What);
  if L > Length(S) then Exit;
  for I:=1 to L do
    if S[I] <> What[I] then Exit;
  Result:=True;
end;

{$IFNDEF V_32}
procedure SetLength(var S: String; ALength: Byte);
begin
  S[0]:=Chr(ALength);
end;

function Trim(const S: String): String;
var
  I, J: Integer;
begin
  I:=1;
  while (I <= Length(S)) and (S[I] <= ' ') do Inc(I);
  J:=Length(S);
  while (J >= 1) and (S[J] <= ' ') do Dec(J);
  Result:=Copy(S, I, J - I + 1)
end;
{$ENDIF}

function MakeStr(C: Char; N: Integer): String;
begin
  if N > 0 then begin
    {$IFDEF V_LONGSTRINGS}
    SetLength(Result, N);
    {$ELSE}
    if N > 255 then N:=255;
    Result[0]:=Chr(N);
    {$ENDIF}
    FillChar(Result[1], Length(Result), C);
  end
  else
    Result:='';
end;

function AddChar(C: Char; const S: String; N: Integer): String;
begin
  if Length(S) < N then
    Result:=MakeStr(C, N - Length(S)) + S
  else
    Result := S;
end;

function ReplaceStr(const Value, FromStr, ToStr: String): String;
var
  I: Integer;
  Source: String;
begin
  Source:=Value;
  Result:='';
  repeat
    I:=Pos(FromStr, Source);
    if I > 0 then begin
      Result:=Result + Copy(Source, 1, I - 1) + ToStr;
      Source:=Copy(Source, I + Length(FromStr), Length(Source));
    end
    else begin
      Result:=Result + Source;
      Break;
    end;
  until False;
end;

procedure ReplaceCharV(var S: String; FromChar, ToChar: Char);
var
  I: Integer;
begin
  for I:=1 to Length(S) do
    if S[I] = FromChar then S[I]:=ToChar;
end;

function ReplaceChar(const S: String; FromChar, ToChar: Char): String;
begin
  Result:=S;
  ReplaceCharV(Result, FromChar, ToChar);
end;

function ContainsChars(const S: String; Chars: TCharSet): Boolean;
var
  I: Integer;
begin
  for I:=1 to Length(S) do
    if S[I] in Chars then begin
      Result:=True;
      Exit;
    end;
  Result:=False;
end;

function DelDupChar(const S: String; C: Char): String;
var
  I, J: Integer;
  CurChar, LastChar: Char;
begin
  SetLength(Result, Length(S));
  if Length(S) > 0 then begin
    LastChar:=S[1];
    Result[1]:=LastChar;
    J:=1;
    for I:=2 to Length(S) do begin
      CurChar:=S[I];
      if (CurChar <> C) or (CurChar <> LastChar) then begin
        Inc(J);
        Result[J]:=CurChar;
        LastChar:=CurChar;
      end;
    end;
    SetLength(Result, J);
  end;
end;

{$IFDEF V_32}
function DecodeCEscapes(const S: String): String;
var
  State: (sNeutral, sWasSlash, sOct, sHex);

  function ConvertToNumber(S: String; var Num: Integer): Boolean;
  var
    I, J, Code: Integer;
  begin
    Result:=False;
    if State = sHex then begin
      Val(S, J, Code);
      if (Code <> 0) or (J > 255) then Exit;
    end
    else begin
      J:=0;
      for I:=1 to Length(S) do begin
        Code:=Ord(S[I]) - Ord('0');
        if (Code < 0) or (Code > 7) then Exit;
        J:=J * 8 + Code;
        if J > 255 then Exit;
      end;
    end;
    Num:=J;
    Result:=True;
  end;

var
  I, J, CharCode: Integer;
  C: Char;
  InP, OutP, InLimit: PChar;
  Number: String;
begin
  I:=Pos('\', S);
  if I = 0 then
    Result:=S
  else begin
    CharCode:=-1;
    State:=sWasSlash;
    J:=I - 1;
    Result:=Copy(S, 1, J);
    InP:=PChar(S) + I;
    I:=Length(S);
    InLimit:=PChar(S) + I;
    SetLength(Result, I);
    OutP:=PChar(Result) + J;
    while InP < InLimit do begin
      C:=InP^;
      Case State of
        sNeutral: begin
          if C <> '\' then begin
            OutP^:=C;
            Inc(OutP);
          end
          else
            State:=sWasSlash;
          Inc(InP);
        end;
        sWasSlash: begin
          Case C of
            '0'..'7': begin
              Number:=C;
              CharCode:=Ord(C) - Ord('0');
              State:=sOct;
              Inc(InP);
              Continue;
            end;
            'a': C:=#7;
            'b': C:=#8;
            'f': C:=#$0C;
            'n': begin
              OutP^:=#$0D;
              Inc(OutP);
              C:=#$0A;
            end;
            'r': C:=#$0D;
            't': C:=#9;
            'v': C:=#$0B;
            '\', '''', '"', '?': ; // C:=C
            'x': begin
              Number:='$';
              State:=sHex;
              Inc(InP);
              Continue;
            end;
          Else begin
            OutP^:='\';
            Inc(OutP);
          end;
          End;
          OutP^:=C;
          Inc(InP);
          Inc(OutP);
          State:=sNeutral;
        end;
        sOct, sHex: begin
          Number:=Number + C;
          if ConvertToNumber(Number, CharCode) then
            Inc(InP)
          else begin
            if CharCode >= 0 then begin
              OutP^:=Chr(CharCode);
              CharCode:=-1;
            end
            else begin
              OutP^:='\';
              if State = sHex then begin
                Inc(OutP);
                OutP^:='x';
              end;
            end;
            Inc(OutP);
            State:=sNeutral;
          end;
        end;
      End;
    end; {while}
    Case State of
      sOct, sHex:
        if CharCode >= 0 then begin
          OutP^:=Chr(CharCode);
          Inc(OutP);
        end;
      sWasSlash: begin
        OutP^:='\';
        Inc(OutP);
      end;
    End;
    SetLength(Result, OutP - PChar(Result));
  end;
end;

function EncodeCEscapes(const S: String; Only7Bit: Boolean): String;
var
  I, L: Integer;
  C, Slash: Char;
  OutP: PChar;
  Hex: String[2];
begin
  L:=Length(S);
  SetLength(Result, 4 * L);
  OutP:=PChar(Result);
  I:=1;
  while I <= L do begin
    C:=S[I];
    Case C of
      #7: Slash:='a';
      #8: Slash:='b';
      #9: Slash:='t';
      #$0B: Slash:='v';
      #$0C: Slash:='f';
      #$0D:
        if (I < L) and (S[I + 1] = #$0A) then begin
          Slash:='n';
          Inc(I);
        end
        else
          Slash:='r';
      '\', '''', '"', '?': Slash:=C;
    Else begin
      Slash:=#0;
      if (C < ' ') or Only7Bit and (C >= #128) then
        if C < #8 then
          Slash:=Chr(Ord(C) + Ord('0'))
        else begin
          OutP^:='\';
          Inc(OutP);
          OutP^:='x';
          Inc(OutP);
          Hex:=IntToHex(Ord(C), 0);
          OutP^:=Hex[1];
          Inc(OutP);
          if Length(Hex) > 1 then begin
            OutP^:=Hex[2];
            Inc(OutP);
          end;
        end
      else begin
        OutP^:=C;
        Inc(OutP);
      end;
    end;
    End;
    if Slash <> #0 then begin
      OutP^:='\';
      Inc(OutP);
      OutP^:=Slash;
      Inc(OutP);
    end;
    Inc(I);
  end; {while}
  SetLength(Result, OutP - PChar(Result));
end;
{$ENDIF}

{$IFDEF WIN32}
function WordPos(SubWord, S: String; CaseSensitive: Boolean): Integer;
var
  LSub, LStr: Integer;
  P, PSub, PStr, Limit: PChar;
begin
  Result:=0;
  LSub:=Length(SubWord);
  LStr:=Length(S);
  if (LSub = 0) or (LSub > LStr) then Exit;
  if not CaseSensitive then begin
    UniqueString(SubWord);
    CharUpperBuff(PChar(SubWord), LSub);
    UniqueString(S);
    CharUpperBuff(PChar(S), LStr);
  end;
  PSub:=PChar(SubWord);
  PStr:=PChar(S);
  P:=PStr;
  Limit:=PStr + LStr;
  repeat
    P:=StrPos(P, PSub);
    if P = nil then Exit;
    if ((P = PStr) or not IsCharAlphaNumeric((P - 1)^)) and
      ((P + LSub >= Limit) or not IsCharAlphaNumeric((P + LSub)^))
    then
      Break;
    Inc(P);
  until P >= Limit;
  Result:=P - PStr + 1;
end;

function AnsiToOEM(const S: String): String;
begin
  SetLength(Result, Length(S));
  if S <> '' then CharToOEM(PChar(S), PChar(Result));
end;

function OEMToAnsi(const S: String): String;
begin
  SetLength(Result, Length(S));
  if S <> '' then OemToChar(PChar(S), PChar(Result));
end;
{$ENDIF}

function IsCorrectIdentifier(const S: String): Boolean;
var
  I: Integer;
begin
  Result:=False;
  if S <> '' then begin
    if S[1] in Digits then Exit;
    for I:=1 to Length(S) do
      if not (S[I] in AlphaNumeric + ['_']) then Exit;
    Result:=True;
  end;
end;

function RemoveComment(const S: String; CommentPrefix: Char): String;
var
  I: Integer;
  C: Char;
begin
  if Pos(CommentPrefix, S) = 0 then
    Result:=Trim(S)
  else begin
    C:=#0;
    for I:=1 to Length(S) do
      if S[I] = CommentPrefix then
        if C = #0 then begin
          Result:=Trim(Copy(S, 1, I - 1));
          Exit;
        end
        else
      else
        if S[I] in ['''', '"'] then
          if C = #0 then C:=S[I]
          else
            if C = S[I] then C:=#0;
    Result:=Trim(S);
  end;
end;

function FStringToLiteral(const S: String; Quote: Char): String;
var
  I, J, K, N: Integer;
  C: Char;
begin
  I:=Pos(Quote, S);
  if I = 0 then
    Result:=Quote + S + Quote
  else begin
    Result:=Quote + Copy(S, 1, I) + Quote;
    N:=Length(S);
    SetLength(Result, N + 2 + NumOfChars(Quote, S));
    K:=I + 3;
    for J:=I + 1 to N do begin
      C:=S[J];
      if C = Quote then begin
        Result[K]:=C;
        Inc(K);
      end;
      Result[K]:=C;
      Inc(K);
    end;
    Result[K]:=Quote;
  end;
end;

function StringToLiteral(const S: String): String;
begin
  Result:=FStringToLiteral(S, '''');
end;

function StringToLiteral2(const S: String): String;
begin
  Result:=FStringToLiteral(S, '"');
end;

{$IFDEF V_32}
function FWideStringToLiteral(const S: WideString; Quote: WideChar): WideString;
var
  I, J, K, N: Integer;
  C: WideChar;
begin
  I:=Pos(Quote, S);
  if I = 0 then
    Result:=Quote + S + Quote
  else begin
    Result:=Quote + Copy(S, 1, I) + Quote;
    N:=Length(S);
    SetLength(Result, N + 2 + NumOfWideChars(Quote, S));
    K:=I + 3;
    for J:=I + 1 to N do begin
      C:=S[J];
      if C = Quote then begin
        Result[K]:=C;
        Inc(K);
      end;
      Result[K]:=C;
      Inc(K);
    end;
    Result[K]:=Quote;
  end;
end;

function WideStringToLiteral(const S: WideString): WideString;
begin
  Result:=FWideStringToLiteral(S, '''');
end;

function WideStringToLiteral2(const S: WideString): WideString;
begin
  Result:=FWideStringToLiteral(S, '"');
end;
{$ENDIF}

function CheckText(const S: String): Boolean;
var
  I: Integer;
begin
  Result:=False;
  if S <> '' then begin
    for I:=1 to Length(S) do
      if S[I] in [#0..' ', '"', '''', #127..#255] then Exit;
    Result:=True;
  end;
end;

{$IFDEF V_32}
function CheckWideText(const S: WideString): Boolean;
var
  I: Integer;
  C: WideChar;
begin
  Result:=False;
  if S <> '' then begin
    for I:=1 to Length(S) do begin
      C:=S[I];
      if (C <= ' ') or (C = '"') or (C = '''') or (C > #127) then Exit;
    end;
    Result:=True;
  end;
end;
{$ENDIF}

function TextToLiteral(const S: String): String;
begin
  if not CheckText(S) then Result:=StringToLiteral(S) else Result:=S;
end;

{$IFDEF V_32}
function WideTextToLiteral(const S: WideString): WideString;
begin
  if not CheckWideText(S) then Result:=WideStringToLiteral(S) else Result:=S;
end;
{$ENDIF}

function TextToLiteral2(const S: String): String;
begin
  if not CheckText(S) then Result:=StringToLiteral2(S) else Result:=S;
end;

function LiteralToString(const S: String): String;
var
  I, J, N: Integer;
  Quote, C: Char;
begin
  N:=Length(S);
  if N >= 2 then begin
    if (S[1] = '''') and (S[N] = '''') then Quote:=''''
    else if (S[1] = '"') and (S[N] = '"') then Quote:='"'
    else begin
      Result:=S;
      Exit;
    end;
    Dec(N);
    SetLength(Result, N - 1);
    I:=2;
    J:=0;
    while I <= N do begin
      C:=S[I];
      Inc(J);
      Result[J]:=C;
      Inc(I);
      if C = Quote then Inc(I);
    end;
    SetLength(Result, J);
  end
  else
    Result:=S;
end;

{$IFDEF V_32}
function LiteralToWideString(const S: WideString): WideString;
var
  I, J, N: Integer;
  Quote, C: WideChar;
begin
  N:=Length(S);
  if N >= 2 then begin
    if (S[1] = '''') and (S[N] = '''') then Quote:=''''
    else if (S[1] = '"') and (S[N] = '"') then Quote:='"'
    else begin
      Result:=S;
      Exit;
    end;
    Dec(N);
    SetLength(Result, N - 1);
    I:=2;
    J:=0;
    while I <= N do begin
      C:=S[I];
      Inc(J);
      Result[J]:=C;
      Inc(I);
      if C = Quote then Inc(I);
    end;
    SetLength(Result, J);
  end
  else
    Result:=S;
end;
{$ENDIF}

function CmpStrF(const S1, S2: String; Flags: LongInt): Integer;
begin
{$IFDEF V_WIN}
{$IFDEF V_LONGSTRINGS}
  if Flags = 0 then Result:=CompareStr(S1, S2)
  else
    Result:=CompareString(Flags and LocaleMask, Flags and FlagsMask, PChar(S1),
      Length(S1), PChar(S2), Length(S2)) - 2;
{$ELSE}
   if Flags and LocaleMask <> 0 then
     if Flags and iCaseInsensitive <> 0 then Result:=AnsiCompareText(S1, S2)
     else Result:=AnsiCompareStr(S1, S2)
   else
     if Flags and iCaseInsensitive <> 0 then Result:=CompareText(S1, S2)
     else Result:=CompareStr(S1, S2)
{$ENDIF}
{$ELSE}
   if Flags and LocaleMask <> 0 then
     if Flags and iCaseInsensitive <> 0 then Result:=AnsiCompareText(S1, S2)
     else Result:=AnsiCompareStr(S1, S2)
   else
     if Flags and iCaseInsensitive <> 0 then Result:=CompareText(S1, S2)
     else Result:=CompareStr(S1, S2)
{$ENDIF}
end;

end.
