{ Version 010629. Copyright  Alexey A.Chernobaev, 1996-2001 }

unit WStrLst;

interface

{$I VCheck.inc}

uses
  SysUtils, ExtType, Vectors, Pointerv, VStream, VectStr, VectErr;

type
  {       ( "Compare" 
    AnsiCompareText) }
  { case-insensitive WideString list ("Compare" method uses AnsiCompareText) }
  TWideStrLst = class(TPointerVector)
  protected
    procedure FreeItems(FromIndex: Integer);
    procedure SetCount(ACount: Integer); override;
    function GetValue(I: Integer): WideString;
    procedure SetValue(I: Integer; const Value: WideString);
    function GetCommaText: WideString;
    function GetCommaText1: WideString;
    procedure FSetCommaText(const Value: WideString; QuoteChar: WideChar);
    procedure SetCommaText(const Value: WideString);
    procedure SetCommaText1(const Value: WideString);
  public
    function CreateCompatibleVector: TPointerVector; override;
    destructor Destroy; override;
    procedure WriteToStream(VStream: TVStream); override;
    procedure ReadFromStream(VStream: TVStream); override;
    procedure Assign(Source: TVector); override;
    function EqualTo(V: TVector): Bool; override;
    procedure Delete(I: Integer); override;
    procedure DeleteRange(I, ACount: Integer); override;
    procedure GetUntyped(I: Integer; var Result); override;
    procedure SetUntyped(I: Integer; const Value); override;
    class function CompareStrings(const S1, S2: WideString): Int32; virtual;
    function Compare(I: Integer; const V): Int32; override;
    procedure SetToDefault; override;
    procedure SetItems(Values: array of WideString);
    {      Values (Count:=High(Values) + 1) }
    { sets the list elements to Values (Count:=High(Values) + 1) }
    procedure Insert(I: Integer; const Value: WideString); virtual;
    {   Value   I }
    { inserts Value in the position I }
    function Add(const Value: WideString): Integer; virtual;
    {          (Count - 1) }
    { appends Value to the end of the list and returns it's index (Count - 1) }
    procedure Move(CurIndex, NewIndex: Integer); virtual;
    {    CurIndex  NewIndex }
    { moves the element from the position CurIndex to NewIndex }
    function IndexFrom(I: Integer; const Value: WideString): Integer; virtual;
    {   ,   I,   Value  ,
       -1,      }
    { returns the index of the first occurrence of Value in the list beginning
      from I or -1 if there's no such occurrence }
    function IndexOf(const Value: WideString): Integer;
    { IndexOf(Value) = IndexFrom(0, Value) }
    function LastIndexFrom(I: Integer; const Value: WideString): Integer; virtual;
    {      Value  , 
        I,  -1,     }
    { returns the index of the last occurrence of Value in the list which is not
      greater then I or -1 if there's no such occurrence }
    function LastIndexOf(const Value: WideString): Integer;
    { LastIndexOf(Value) = LastIndexFrom(Count - 1, Value) }
    function Remove(const Value: WideString): Integer;
    {    Value  ,    Delete 
         ,  -1,  Value   }
    { searches for the first occurrence of Value in the list, deletes it with
      Delete and returns the index of the deleted value or -1 if Value wasn't
      found }
    function RemoveLast(const Value: WideString): Integer;
    {    Value  ,    Delete 
         ,  -1,  Value   }
    { searches for the last occurrence of Value in the list, deletes it with
      Delete and returns the index of the deleted value or -1 if Value wasn't
      found }
    function RemoveFrom(I: Integer; const Value: WideString): Integer;
    {  ,   I,  Value  ,   
      Delete     ,  -1,  Value
        }
    { searches for the first occurrence of Value in the list beginning from I,
      deletes it with Delete and returns the index of the deleted value or -1 if
      Value wasn't found }
    function RemoveLastFrom(I: Integer; const Value: WideString): Integer;
    {  ,    I,  Value     
       Delete,    ,  -1,  Value
        }
    { searches for the last occurrence of Value in the list which is not greater
      then I, deletes it with Delete and returns the index of the deleted value
      or -1 if Value wasn't found }
    function NumberOfValues(const Value: WideString): Integer;
    {   ,  Value }
    { returns the number of elements equal to Value }
    function FindInSortedRange(const Value: WideString; L, H: Integer): Integer;
    {    Value    
      ,    L   H;   
       ,  -1,     }
    { searches for the Value in the sorted (ascending) list dichotomically
      from the index L to H; returns the minimum index of Value or -1 if Value
      wasn't found }
    function FindInSorted(const Value: WideString): Integer;
    {   Value      ;
          ,  -1,  
        }
    { searches for the Value in the sorted (ascending) list dichotomically;
      returns the minimum index of Value or -1 if Value wasn't found }
    function FindInsertPosition(const Value: WideString; L, H: Integer;
      var Index: Integer): Bool;
    {   Value      ,
         L   H;  True,   
      (  Index     ), 
       False (  Index ,    Value,
         ) }
    { searches for the Value in the sorted (ascending) list dichotomically
      from the index L to H; returns True if Value was found (in such case Index
      is equal to the minimum index of Value), otherwise returns False (in such
      case Index is equal to the position where Value can be inserted so that
      the list remains sorted) }
    function Find(const Value: WideString; var Index: Integer): Bool;
    {  FindInsertPosition   }
    { analog of FindInsertPosition for the whole list }
    procedure AddStrings(List: TWideStrLst);
    {   List }
    { adds strings List }
    function Last: WideString;
    {     (    ) }
    { returns the last element of the list (which must be non-empty) }
    function Pop: WideString;
    {     (    )
         (..     ) }
    { returns the last element of the list (which must be non-empty) and removes
      it (i.e. decreases the length of the list by one) }
    property CommaText: WideString read GetCommaText write SetCommaText;
    {     ,     ,
              }
    { presents the list as a single comma-delimited WideString where the list
      elements are separated with commas and enclosed in the double quotes }
    property CommaText1: WideString read GetCommaText1 write SetCommaText1;
    {     ,     ,
             ,   
       ,    ,     
      (. CheckText   VectStr) }
    { presents the list as a single comma-delimited WideString where the list
      elements are separated with commas and enclosed in the single quotes if
      they contain characters other then the Latin letters, digits and several
      other characters (see ChectText in the unit VectStr) }
    property Items[I: Integer]: WideString read GetValue write SetValue; default;
  end;

  TWideStrLstClass = class of TWideStrLst;

  TSortedWideStrLst = class(TWideStrLst)
    function CreateCompatibleVector: TPointerVector; override;
    procedure Insert(I: Integer; const Value: WideString); override;
    function Add(const Value: WideString): Integer; override;
    procedure Move(CurIndex, NewIndex: Integer); override;
    function IndexFrom(I: Integer; const Value: WideString): Integer; override;
    function LastIndexFrom(I: Integer; const Value: WideString): Integer; override;
  end;

  TSortedWideStrLstClass = class of TSortedWideStrLst;

implementation

function NewWideStr(const S: WideString): PWideString;
const
  EmptyWideStr: WideString = '';
  NullWideStr: PWideString = @EmptyWideStr;
begin
  if S = '' then
    Result:=NullWideStr
  else begin
    New(Result);
    Result^:=S;
  end;
end;

procedure DisposeWideStr(P: PWideString);
begin
  if (P <> nil) and (P^ <> '') then Dispose(P);
end;

{ TWideStrLst }

procedure TWideStrLst.FreeItems(FromIndex: Integer);
var
  I: Integer;
begin
  for I:=FromIndex to Count - 1 do Items[I]:='';
end;

procedure TWideStrLst.SetCount(ACount: Integer);
begin
  if ACount < Count then FreeItems(ACount);
  inherited SetCount(ACount);
end;

function TWideStrLst.GetValue(I: Integer): WideString;
var
  P: PWideString;
begin
  {$IFDEF CHECK_VECTORS}
  if (I < 0) or (I >= Count) then Error(SRangeError, [I]);
  {$ENDIF}
  P:=PPointerArray(FItems)^[I];
  if P <> nil then Result:=P^ else Result:='';
end;

procedure TWideStrLst.SetValue(I: Integer; const Value: WideString);
begin
  {$IFDEF CHECK_VECTORS}
  if (I < 0) or (I >= Count) then Error(SRangeError, [I]);
  {$ENDIF}
  DisposeWideStr(PPointerArray(FItems)^[I]);
  PPointerArray(FItems)^[I]:=NewWideStr(Value);
end;

function TWideStrLst.GetCommaText: WideString;
var
  I: Integer;
begin
  Result:='';
  if Count > 0 then begin
    Result:=WideStringToLiteral2(Items[0]);
    for I:=1 to Count - 1 do
      Result:=Result + ',' + WideStringToLiteral2(Items[I]);
  end;
end;

function TWideStrLst.GetCommaText1: WideString;
var
  I: Integer;
begin
  Result:='';
  if Count > 0 then begin
    Result:=WideTextToLiteral(Items[0]);
    for I:=1 to Count - 1 do
      Result:=Result + ', ' + WideTextToLiteral(Items[I]);
  end;
end;

procedure TWideStrLst.FSetCommaText(const Value: WideString; QuoteChar: WideChar);
var
  I: Integer;
  C: WideChar;
  Quote: Bool;
  S: WideString;
begin
  Clear;
  S:='';
  Quote:=False;
  for I:=1 to Length(Value) do begin
    C:=Value[I];
    if Quote or (C <> ',') then begin
      if C = QuoteChar then Quote:=not Quote;
      S:=S + C;
    end
    else begin
      Add(LiteralToWideString(S));
      S:='';
    end;
  end;
  if S <> '' then Add(LiteralToWideString(S));
end;

procedure TWideStrLst.SetCommaText(const Value: WideString);
begin
  FSetCommaText(Value, '"');
end;

procedure TWideStrLst.SetCommaText1(const Value: WideString);
begin
  FSetCommaText(Value, '''');
end;

function TWideStrLst.CreateCompatibleVector: TPointerVector;
begin
  Result:=TWideStrLst.Create;
end;

destructor TWideStrLst.Destroy;
begin
  SetToDefault;
  inherited Destroy;
end;

procedure TWideStrLst.WriteToStream(VStream: TVStream);
var
  I: Integer;
begin
  VStream.WriteInt32(FCount);
  for I:=0 to FCount - 1 do VStream.WriteWideString(Items[I]);
end;

procedure TWideStrLst.ReadFromStream(VStream: TVStream);
var
  I: Integer;
begin
  Clear;
  for I:=0 to VStream.ReadInt32 - 1 do Add(VStream.ReadWideString);
end;

procedure TWideStrLst.Delete(I: Integer);
begin
  Items[I]:='';
  inherited Delete(I);
end;

procedure TWideStrLst.DeleteRange(I, ACount: Integer);
var
  J: Integer;
begin
  for J:=I to I + ACount - 1 do
    Items[J]:='';
  inherited DeleteRange(I, ACount);
end;

procedure TWideStrLst.GetUntyped(I: Integer; var Result);
begin
  PWideString(Result):=inherited GetValue(I);
end;

procedure TWideStrLst.SetUntyped(I: Integer; const Value);
begin
  Items[I]:=PWideString(Value)^;
end;

class function TWideStrLst.CompareStrings(const S1, S2: WideString): Int32;
begin
  if S1 < S2 then Result:=-1
  else if S1 > S2 then Result:=1
  else Result:=0;
end;

function TWideStrLst.Compare(I: Integer; const V): Int32;
var
  S: WideString;
begin
  if PWideString(V) <> nil then
    S:=PWideString(V)^
  else
    S:='';
  Result:=CompareStrings(Items[I], S);
end;

procedure TWideStrLst.SetToDefault;
begin
  FreeItems(0);
end;

procedure TWideStrLst.SetItems(Values: array of WideString);
var
  I: Integer;
begin
  Count:=High(Values) + 1;
  for I:=0 to High(Values) do
    Items[I]:=Values[I];
end;

procedure TWideStrLst.Insert(I: Integer; const Value: WideString);
var
  P: PWideString;
begin
  P:=NewWideStr(Value);
  try
    inherited Insert(I, P);
  except
    DisposeWideStr(P);
    raise;
  end; 
end;

function TWideStrLst.Add(const Value: WideString): Integer;
begin
  Result:=Count;
  Insert(Result, Value);
end;

procedure TWideStrLst.Move(CurIndex, NewIndex: Integer);
var
  T: WideString;
begin
  if CurIndex <> NewIndex then begin
    T:=Items[CurIndex];
    Delete(CurIndex);
    Insert(NewIndex, T);
  end;
end;

function TWideStrLst.IndexFrom(I: Integer; const Value: WideString): Integer;
var
  ACount: Integer;
  P: PWideString;
begin
  Result:=I;
  ACount:=Count;
  P:=NewWideStr(Value);
  try
    while Result < ACount do
      if Compare(Result, P) = 0 then Exit else Inc(Result);
    Result:=-1;
  finally
    DisposeWideStr(P);
  end;
end;

function TWideStrLst.IndexOf(const Value: WideString): Integer;
begin
  Result:=IndexFrom(0, Value);
end;

function TWideStrLst.LastIndexFrom(I: Integer; const Value: WideString): Integer;
var
  P: PWideString;
begin
  P:=NewWideStr(Value);
  try
    Result:=I;
    while Result >= 0 do
      if Compare(Result, P) = 0 then Exit else Dec(Result);
  finally
    DisposeWideStr(P);
  end;
end;

function TWideStrLst.LastIndexOf(const Value: WideString): Integer;
begin
  Result:=LastIndexFrom(Count - 1, Value);
end;

function TWideStrLst.Remove(const Value: WideString): Integer;
begin
  Result:=IndexOf(Value);
  if Result >= 0 then Delete(Result);
end;

function TWideStrLst.RemoveLast(const Value: WideString): Integer;
begin
  Result:=LastIndexOf(Value);
  if Result >= 0 then Delete(Result);
end;

function TWideStrLst.RemoveFrom(I: Integer; const Value: WideString): Integer;
begin
  Result:=IndexFrom(I, Value);
  if Result >= 0 then Delete(Result);
end;

function TWideStrLst.RemoveLastFrom(I: Integer; const Value: WideString): Integer;
begin
  Result:=LastIndexFrom(I, Value);
  if Result >= 0 then Delete(Result);
end;

function TWideStrLst.NumberOfValues(const Value: WideString): Integer;
var
  I: Integer;
  P: PWideString;
begin
  P:=NewWideStr(Value);
  try
    Result:=0;
    for I:=0 to Count - 1 do
      if Compare(I, P) = 0 then Inc(Result);
  finally
    DisposeWideStr(P);
  end;
end;

function TWideStrLst.FindInsertPosition(const Value: WideString; L, H: Integer;
  var Index: Integer): Bool;
var
  I, C: Integer;
  P: PWideString;
begin
  Result:=False;
  P:=NewWideStr(Value);
  try
    while L <= H do begin
      I:=(L + H) shr 1;
      C:=Compare(I, P);
      if C < 0 then L:=I + 1
      else begin
        H:=I - 1;
        if C = 0 then begin
          Result:=True;
          L:=I;
        end;
      end;
    end;
  finally
    DisposeWideStr(P);
  end;
  Index:=L;
end;

function TWideStrLst.Find(const Value: WideString; var Index: Integer): Bool;
begin
  Result:=FindInsertPosition(Value, 0, FCount - 1, Index);
end;

function TWideStrLst.FindInSortedRange(const Value: WideString; L, H: Integer): Integer;
begin
  if not FindInsertPosition(Value, L, H, Result) then Result:=-1;
end;

function TWideStrLst.FindInSorted(const Value: WideString): Integer;
begin
  Result:=FindInSortedRange(Value, 0, FCount - 1);
end;

procedure TWideStrLst.Assign(Source: TVector);
var
  I: Integer;
begin
  if Source is TWideStrLst then begin
    Count:=Source.Count;
    for I:=0 to Count - 1 do
      Items[I]:=TWideStrLst(Source).Items[I];
  end
  else
    Error(SAssignError, [0]);
end;

{$IFDEF NOWARN}{$WARNINGS OFF}{$ENDIF}
function TWideStrLst.EqualTo(V: TVector): Bool;
var
  I: Integer;
begin
  if V is TWideStrLst then begin
    if FCount = V.Count then
      for I:=0 to FCount - 1 do
        if Compare(I, PPointerArray(TWideStrLst(V).FItems)^[I]) <> 0 then begin
          Result:=False;
          Exit;
        end;
    Result:=True;
  end
  else
    Error(SIncompatibleClasses, [0]);
end;
{$IFDEF NOWARN}{$WARNINGS ON}{$ENDIF}

procedure TWideStrLst.AddStrings(List: TWideStrLst);
var
  I: Integer;
begin
  for I:=0 to List.Count - 1 do Add(List.Items[I]);
end;

function TWideStrLst.Last: WideString;
begin
  Result:=Items[Count - 1];
end;

function TWideStrLst.Pop: WideString;
var
  N: Integer;
begin
  N:=Count - 1;
  Result:=Items[N];
  Count:=N;
end;

{ TSortedWideStrLst }

function TSortedWideStrLst.CreateCompatibleVector: TPointerVector;
begin
  Result:=TSortedWideStrLst.Create;
end;

procedure TSortedWideStrLst.Insert(I: Integer; const Value: WideString);
begin
  Error(SMethodNotApplicable, [0]);
end;

function TSortedWideStrLst.Add(const Value: WideString): Integer;
begin
  if FindInsertPosition(Value, 0, FCount - 1, Result) then
    Error(SDuplicateError, [0])
  else
    inherited Insert(Result, Value);
end;

procedure TSortedWideStrLst.Move(CurIndex, NewIndex: Integer);
begin
  Error(SMethodNotApplicable, [0]);
end;

function TSortedWideStrLst.IndexFrom(I: Integer; const Value: WideString): Integer;
begin
  Result:=FindInSorted(Value);
end;

function TSortedWideStrLst.LastIndexFrom(I: Integer; const Value: WideString): Integer;
begin
  Result:=FindInSorted(Value);
end;

end.
