////////////////////////////////////////////////////////////////////////////////
// Jazarsoft Wizard                                                           //
////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// VERSION      : 1.0                                                         //
// AUTHOR       : James Azarja                                                //
// CREATED      : 20 September 2001                                           //
// WEBSITE      : http://www.jazarsoft.com                                    //
// FORUM        : http://www.jazarsoft.com/vcl/forum/                         //
// LEGAL        : Copyright (C) 2001 Jazarsoft.                               //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// This code may be used and modified by anyone so long as  this header and   //
// copyright  information remains intact.                                     //
//                                                                            //
// The code is provided "as-is" and without warranty of any kind,             //
// expressed, implied or otherwise, including and without limitation, any     //
// warranty of merchantability or fitness for a  particular purpose.         //
//                                                                            //
// In no event shall the author be liable for any special, incidental,        //
// indirect or consequential damages whatsoever (including, without           //
// limitation, damages for loss of profits, business interruption, loss       //
// of information, or any other loss), whether or not advised of the          //
// possibility of damage, and on any theory of liability, arising out of      //
// or in connection with the use or inability to use this software.         //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////
//                                                                            //
// HISTORY :                                                                  //
//                                                                            //
// 1.0 :                                                                      //
//   Initial development                                                      //
////////////////////////////////////////////////////////////////////////////////

unit Wizard;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, ImgList, StdCtrls;

type
  TOnPageChanged = procedure(Sender: TObject;Index:Integer;Title:String) of object;
  
  TWizard = class(TComponent)
  private
    FPages      : tNoteBook;
    FPageSteps  : tStrings;
    FImages     : tImageList;
    FImageSteps : tStrings;
    FSideImage  : tImage;
    FNextButton,
    FBackButton,
    FCancelButton: tButton;

    FOnNext,
    FOnBack     : tOnPageChanged;
    FOnFinish,
    FOnCancel   : tNotifyEvent;
    
    Procedure SetPages(Value:TNoteBook);
    Procedure SetImages(Value:TImageList);
    Procedure SetPageSteps(Value:tStrings);
    Procedure SetImageSteps(Value:tStrings);
    Procedure SetNextButton(Value:tButton);
    Procedure SetBackButton(Value:tButton);
    Procedure SetCancelButton(Value:tButton);
  protected
    Procedure GoNext(Sender: TObject);
    procedure GoBack(Sender: TObject);
    Procedure GoCancel(Sender: TObject);
    Procedure UpdateScreen;
  public
    Constructor Create(AOwner:TComponent);override;
    Destructor Destroy;override;
    Procedure Loaded;override;
  published
    property Pages : TNoteBook         Read FPages          Write SetPages;
    property PageSteps : tStrings      Read FPageSteps      Write SetPageSteps;
    property Images : TImageList       Read FImages         Write SetImages;
    property ImageSteps : tStrings     Read FImageSteps     Write SetImageSteps;
    property SideImage : tImage        Read FSideImage      Write FSideImage;
    property NextButton : tButton      Read FNextButton     Write SetNextButton;
    property BackButton : tButton      Read FBackButton     Write SetBackButton;
    property CancelButton : tButton    Read FCancelButton   Write SetCancelButton;

    property OnNext : TOnPageChanged   Read FOnNext         Write FOnNext;
    property OnBack : TOnPageChanged   Read FOnBack         Write FOnBack;
    property OnCancel: TNotifyEvent    Read FOnCancel       Write FOnCancel;    
    property OnFinish: TNotifyEvent    Read FOnFinish       Write FOnFinish;    
  end;

procedure Register;

implementation

Constructor TWizard.Create(AOwner:TComponent);
Begin
  inherited;
  FPageSteps := tStringList.Create;
  FImageSteps := tStringList.Create;
End;

Destructor TWizard.Destroy;
Begin
  FImageSteps.Free;
  FPageSteps.Free;
  inherited;
End;

Procedure TWizard.Loaded;
Begin
  inherited;
  FPages.ActivePage:=FPageSteps[0];
  UpdateScreen;
End;

Procedure TWizard.SetPages(Value:TNoteBook);
Var Count:Integer;
Begin
  If FPages<>Value then
  Begin
    FPages:=Value;
    If Assigned(Value) then
    Begin
      If csDesigning in ComponentState then
      Begin
       FPageSteps.Clear;
       For Count:=1 to Value.Pages.Count do
         FPageSteps.Add(Value.Pages[Count-1]);
      End;
    End;
  End;
End;

Procedure TWizard.SetImages(Value:TImageList);
Begin
  If FImages<>Value then
  Begin
    FImages:=Value;
    If csDesigning in ComponentState then
    Begin
     FImageSteps.Clear;
    End;
  End;
End;

Procedure TWizard.SetPageSteps(Value:tStrings);
Begin
  If Value<>FPageSteps then
  Begin
    FPageSteps.Assign(Value);
  End;
End;

Procedure TWizard.SetImageSteps(Value:tStrings);
Begin
  If Value<>FImageSteps then
  Begin
    FImageSteps.Assign(Value);
  End;
End;

Procedure TWizard.SetNextButton(Value:tButton);
Begin
  If FNextButton<>Value then
  Begin
    FNextButton:=Value;
    FNextButton.OnClick:=GoNext;
    UpdateScreen; 
  End;
End;

Procedure TWizard.SetBackButton(Value:tButton);
Begin
  If FBackButton<>Value then
  Begin
    FBackButton:=Value;
    FBackButton.OnClick:=GoBack;
    UpdateScreen;
  End;
End;

Procedure TWizard.SetCancelButton(Value:tButton);
Begin
  If FCancelButton<>Value then
  Begin
    FCancelButton:=Value;
    FCancelButton.OnClick:=GoCancel;
    UpdateScreen; 
  End;
End;

Procedure TWizard.GoNext(Sender: TObject);
Var CurPos : Integer;
Begin
  CurPos:=FPageSteps.IndexOf(FPages.ActivePage);
  If CurPos<>FPageSteps.Count-1 then
  Begin
   FPages.ActivePage:=FPageSteps[CurPos+1];
   If Assigned(FOnNext) then
     FOnNext(Self, CurPos, FPages.ActivePage); 
  end else
   Begin
    { End of Wizard }
    If Assigned(FOnFinish) then
      FOnFinish(Self);
   End;
  UpdateScreen; 
End;

Procedure TWizard.GoBack(Sender: TObject);
Var CurPos : Integer;
begin
  CurPos:=FPageSteps.IndexOf(FPages.ActivePage);
  If CurPos<>0 then
  Begin
   FPages.ActivePage:=FPageSteps[CurPos-1];
   If Assigned(FOnBack) then
     FOnBack(Self, CurPos, FPages.ActivePage); 
  End;
  UpdateScreen;
End;

Procedure TWizard.GoCancel(Sender: TObject);
Begin
  If Assigned(FOnCancel) then
    FOnCancel(Self);
End;

Procedure TWizard.UpdateScreen;
Var Buffer : tBitmap;
Begin
  If Assigned(FCancelButton) then
  Begin
   FCancelButton.Caption:='&Cancel';
  End;

  If Assigned(FBackButton) And Assigned(FPages) then
  Begin
   FBackButton.Enabled:=FPageSteps.IndexOf(FPages.ActivePage)>0;
   FBackButton.Caption:='< &Back';
  End;

  If Assigned(FNextButton) And Assigned(FPages) then
   If FPageSteps.IndexOf(FPages.ActivePage)=FPageSteps.Count-1 then
     FNextButton.Caption:='&Finish' else FNextButton.Caption:='&Next >';

  If Assigned(FSideImage) and Assigned(FImages) then
  Begin
    Buffer := tBitmap.Create;

    if FImageSteps.IndexOf(FPages.ActivePage)<>-1 then
    Begin
      if FImageSteps.IndexOf(FPages.ActivePage) < FImages.Count then
       FImages.GetBitmap(FImageSteps.IndexOf(FPages.ActivePage),Buffer) else
       FImages.GetBitmap(0,Buffer);
    End else
    Begin
      FImages.GetBitmap(0,Buffer);
    End;
    FSideImage.Picture.Bitmap.Assign(Buffer);

    Buffer.Free;
  End;
End;

procedure Register;
begin
  RegisterComponents('Jazarsoft', [TWizard]);
end;

end.
