unit PixelFont;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Graphics;

type
  TLetter = record
    Letter: Char;
    Width: Byte;
    Data: String;
  end;

  TPixelFont = class(TGraphicControl)
  private
    FCaption: string;
    FColor: TColor;
    FSizeFactor: Integer;
    procedure SetCaption(const Value: string);
    procedure SetColor(const Value: TColor);
    function GetIDFromChar(aChar: Char): Integer;
    procedure SetSizeFactor(const Value: Integer);
    procedure SetPixel(X, Y, aSizeFactor: Integer; Color: TColor);    
    { Private declarations }
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure SaveToFile(Filename: TFileName);
  published
    property Caption: string read FCaption write SetCaption;
    property Color: TColor read FColor write SetColor default clBlack;
    property SizeFactor: Integer read FSizeFactor write SetSizeFactor;
    property onClick;
  end;

procedure Register;

var
  Letters: Array of TLetter;

implementation

procedure Register;
begin
  RegisterComponents('Standard', [TPixelFont]);
end;

{ TPixelFont }

constructor TPixelFont.Create(AOwner: TComponent);
begin
  inherited;
  SetLength(Letters, 70);

  {Creation of letters}

  Letters[0].Width := 4;
  Letters[0].Data := '1111' +
                     '1001' +
                     '1001' +
                     '1001' +
                     '1111';


  Letters[1].Letter := 'A';
  Letters[1].Width := 4;
  Letters[1].Data := '0110' +
                     '1001' +
                     '1111' +
                     '1001' +
                     '1001';


  Letters[2].Letter := 'B';
  Letters[2].Width := 4;
  Letters[2].Data := '1110' +
                     '1001' +
                     '1110' +
                     '1001' +
                     '1110';


  Letters[3].Letter := 'C';
  Letters[3].Width := 4;
  Letters[3].Data := '0110' +
                     '1001' +
                     '1000' +
                     '1001' +
                     '0110';


  Letters[4].Letter := 'D';
  Letters[4].Width := 4;
  Letters[4].Data := '1110' +
                     '1001' +
                     '1001' +
                     '1001' +
                     '1110';


  Letters[5].Letter := 'E';
  Letters[5].Width := 3;
  Letters[5].Data := '111' +
                     '100' +
                     '110' +
                     '100' +
                     '111';


  Letters[6].Letter := 'F';
  Letters[6].Width := 3;
  Letters[6].Data := '111' +
                     '100' +
                     '110' +
                     '100' +
                     '100';


  Letters[7].Letter := 'G';
  Letters[7].Width := 4;
  Letters[7].Data := '0111' +
                     '1000' +
                     '1011' +
                     '1001' +
                     '0111';


  Letters[8].Letter := 'H';
  Letters[8].Width := 4;
  Letters[8].Data := '1001' +
                     '1001' +
                     '1111' +
                     '1001' +
                     '1001';


  Letters[9].Letter := 'I';
  Letters[9].Width := 3;
  Letters[9].Data := '111' +
                     '010' +
                     '010' +
                     '010' +
                     '111';


  Letters[10].Letter := 'J';
  Letters[10].Width := 4;
  Letters[10].Data := '0001' +
                      '0001' +
                      '0001' +
                      '1001' +
                      '0110';


  Letters[11].Letter := 'K';
  Letters[11].Width := 4;
  Letters[11].Data := '1001' +
                      '1010' +
                      '1100' +
                      '1010' +
                      '1001';


  Letters[12].Letter := 'L';
  Letters[12].Width := 3;
  Letters[12].Data := '100' +
                      '100' +
                      '100' +
                      '100' +
                      '111';


  Letters[13].Letter := 'M';
  Letters[13].Width := 5;
  Letters[13].Data := '10001' +
                      '11011' +
                      '10101' +
                      '10001' +
                      '10001';


  Letters[14].Letter := 'N';
  Letters[14].Width := 4;
  Letters[14].Data := '1001' +
                      '1101' +
                      '1011' +
                      '1001' +
                      '1001';


  Letters[15].Letter := 'O';
  Letters[15].Width := 4;
  Letters[15].Data := '0110' +
                      '1001' +
                      '1001' +
                      '1001' +
                      '0110';


  Letters[16].Letter := 'P';
  Letters[16].Width := 4;
  Letters[16].Data := '1110' +
                      '1001' +
                      '1110' +
                      '1000' +
                      '1000';


  Letters[17].Letter := 'Q';
  Letters[17].Width := 4;
  Letters[17].Data := '0110' +
                      '1001' +
                      '1001' +
                      '1010' +
                      '0111';


  Letters[18].Letter := 'R';
  Letters[18].Width := 4;
  Letters[18].Data := '1110' +
                      '1001' +
                      '1110' +
                      '1010' +
                      '1001';


  Letters[19].Letter := 'S';
  Letters[19].Width := 4;
  Letters[19].Data := '0110' +
                      '1000' +
                      '0110' +
                      '0001' +
                      '0110';


  Letters[20].Letter := 'T';
  Letters[20].Width := 3;
  Letters[20].Data := '111' +
                      '010' +
                      '010' +
                      '010' +
                      '010';


  Letters[21].Letter := 'U';
  Letters[21].Width := 4;
  Letters[21].Data := '1001' +
                      '1001' +
                      '1001' +
                      '1001' +
                      '0110';


  Letters[22].Letter := 'V';
  Letters[22].Width := 5;
  Letters[22].Data := '10001' +
                      '10001' +
                      '01010' +
                      '01010' +
                      '00100';


  Letters[23].Letter := 'W';
  Letters[23].Width := 5;
  Letters[23].Data := '10001' +
                      '10001' +
                      '10101' +
                      '10101' +
                      '01010';


  Letters[24].Letter := 'X';
  Letters[24].Width := 3;
  Letters[24].Data := '101' +
                      '101' +
                      '010' +
                      '101' +
                      '101';


  Letters[25].Letter := 'Y';
  Letters[25].Width := 3;
  Letters[25].Data := '101' +
                      '101' +
                      '010' +
                      '010' +
                      '010';


  Letters[26].Letter := 'Z';
  Letters[26].Width := 4;
  Letters[26].Data := '1111' +
                      '0010' +
                      '0100' +
                      '1000' +
                      '1111';


  Letters[27].Letter := ' ';
  Letters[27].Width := 3;
  Letters[27].Data := '000' +
                      '000' +
                      '000' +
                      '000' +
                      '000';


  Letters[28].Letter := '!';
  Letters[28].Width := 1;
  Letters[28].Data := '1' +
                      '1' +
                      '1' +
                      '0' +
                      '1';


  Letters[29].Letter := '@';
  Letters[29].Width := 5;
  Letters[29].Data := '01110' +
                      '10101' +
                      '10111' +
                      '10000' +
                      '01110';


  Letters[30].Letter := '#';
  Letters[30].Width := 5;
  Letters[30].Data := '01010' +
                      '11111' +
                      '01010' +
                      '11111' +
                      '01010';


  Letters[31].Letter := '$';
  Letters[31].Width := 5;
  Letters[31].Data := '01110' +
                      '10100' +
                      '01110' +
                      '00101' +
                      '01110';


  Letters[32].Letter := '%';
  Letters[32].Width := 5;
  Letters[32].Data := '11001' +
                      '11010' +
                      '00100' +
                      '01011' +
                      '10011';


  Letters[33].Letter := '^';
  Letters[33].Width := 3;
  Letters[33].Data := '010' +
                      '101' +
                      '000' +
                      '000' +
                      '000';


  Letters[34].Letter := '&';
  Letters[34].Width := 4;
  Letters[34].Data := '0100' +
                      '1010' +
                      '0100' +
                      '1010' +
                      '0101';


  Letters[35].Letter := '*';
  Letters[35].Width := 2;
  Letters[35].Data := '11' +
                      '11' +
                      '00' +
                      '00' +
                      '00';


  Letters[36].Letter := '(';
  Letters[36].Width := 2;
  Letters[36].Data := '01' +
                      '10' +
                      '10' +
                      '10' +
                      '01';


  Letters[37].Letter := ')';
  Letters[37].Width := 2;
  Letters[37].Data := '10' +
                      '01' +
                      '01' +
                      '01' +
                      '10';


  Letters[38].Letter := ',';
  Letters[38].Width := 2;
  Letters[38].Data := '00' +
                      '00' +
                      '00' +
                      '01' +
                      '10';


  Letters[39].Letter := '.';
  Letters[39].Width := 1;
  Letters[39].Data := '0' +
                      '0' +
                      '0' +
                      '0' +
                      '1';


  Letters[40].Letter := '/';
  Letters[40].Width := 3;
  Letters[40].Data := '001' +
                      '001' +
                      '010' +
                      '100' +
                      '100';


  Letters[41].Letter := '<';
  Letters[41].Width := 3;
  Letters[41].Data := '001' +
                      '010' +
                      '100' +
                      '010' +
                      '001';


  Letters[42].Letter := '>';
  Letters[42].Width := 3;
  Letters[42].Data := '100' +
                      '010' +
                      '001' +
                      '010' +
                      '100';


  Letters[43].Letter := '?';
  Letters[43].Width := 3;
  Letters[43].Data := '110' +
                      '001' +
                      '010' +
                      '000' +
                      '010';


  Letters[44].Letter := ';';
  Letters[44].Width := 2;
  Letters[44].Data := '00' +
                      '01' +
                      '00' +
                      '01' +
                      '10';


  Letters[45].Letter := '''';
  Letters[45].Width := 1;
  Letters[45].Data := '1' +
                      '1' +
                      '0' +
                      '0' +
                      '0';


  Letters[46].Letter := ':';
  Letters[46].Width := 1;
  Letters[46].Data := '0' +
                      '1' +
                      '0' +
                      '1' +
                      '0';


  Letters[47].Letter := '"';
  Letters[47].Width := 3;
  Letters[47].Data := '101' +
                      '101' +
                      '000' +
                      '000' +
                      '000';


  Letters[48].Letter := '|';
  Letters[48].Width := 1;
  Letters[48].Data := '1' +
                      '1' +
                      '1' +
                      '1' +
                      '1';


  Letters[49].Letter := '[';
  Letters[49].Width := 2;
  Letters[49].Data := '11' +
                      '10' +
                      '10' +
                      '10' +
                      '11';


  Letters[50].Letter := ']';
  Letters[50].Width := 2;
  Letters[50].Data := '11' +
                      '01' +
                      '01' +
                      '01' +
                      '11';


  Letters[51].Letter := '{';
  Letters[51].Width := 3;
  Letters[51].Data := '011' +
                      '010' +
                      '100' +
                      '010' +
                      '011';


  Letters[52].Letter := '}';
  Letters[52].Width := 3;
  Letters[52].Data := '110' +
                      '010' +
                      '001' +
                      '010' +
                      '110';


  Letters[53].Letter := '-';
  Letters[53].Width := 3;
  Letters[53].Data := '000' +
                      '000' +
                      '111' +
                      '000' +
                      '000';


  Letters[54].Letter := '=';
  Letters[54].Width := 3;
  Letters[54].Data := '000' +
                      '111' +
                      '000' +
                      '111' +
                      '000';

  Letters[55].Letter := '_';
  Letters[55].Width := 4;
  Letters[55].Data := '0000' +
                      '0000' +
                      '0000' +
                      '0000' +
                      '1111';


  Letters[56].Letter := '+';
  Letters[56].Width := 3;
  Letters[56].Data := '000' +
                      '010' +
                      '111' +
                      '010' +
                      '000';


  Letters[57].Letter := '1';
  Letters[57].Width := 3;
  Letters[57].Data := '010' +
                      '110' +
                      '010' +
                      '010' +
                      '111';


  Letters[58].Letter := '2';
  Letters[58].Width := 4;
  Letters[58].Data := '0110' +
                      '1001' +
                      '0010' +
                      '0100' +
                      '1111';


  Letters[59].Letter := '3';
  Letters[59].Width := 4;
  Letters[59].Data := '1111' +
                      '0010' +
                      '0110' +
                      '0001' +
                      '1110';


  Letters[60].Letter := '4';
  Letters[60].Width := 4;
  Letters[60].Data := '0010' +
                      '0110' +
                      '1111' +
                      '0010' +
                      '0010';


  Letters[61].Letter := '5';
  Letters[61].Width := 3;
  Letters[61].Data := '111' +
                      '100' +
                      '111' +
                      '001' +
                      '110';


  Letters[62].Letter := '6';
  Letters[62].Width := 4;
  Letters[62].Data := '0110' +
                      '1000' +
                      '1110' +
                      '1001' +
                      '0110';


  Letters[63].Letter := '7';
  Letters[63].Width := 3;
  Letters[63].Data := '111' +
                      '001' +
                      '001' +
                      '010' +
                      '010';


  Letters[64].Letter := '8';
  Letters[64].Width := 4;
  Letters[64].Data := '0110' +
                      '1001' +
                      '0110' +
                      '1001' +
                      '0110';


  Letters[65].Letter := '9';
  Letters[65].Width := 4;
  Letters[65].Data := '0110' +
                      '1001' +
                      '0111' +
                      '0001' +
                      '0110';


  Letters[66].Letter := '0';
  Letters[66].Width := 3;
  Letters[66].Data := '010' +
                      '101' +
                      '101' +
                      '101' +
                      '010';


  Letters[67].Letter := '';
  Letters[67].Width := 5;
  Letters[67].Data := '01110' +
                      '10011' +
                      '10101' +
                      '11001' +
                      '01110';


  Letters[68].Letter := '';
  Letters[68].Width := 5;
  Letters[68].Data := '01111' +
                      '10010' +
                      '11111' +
                      '10010' +
                      '10011';


  Letters[69].Letter := '';
  Letters[69].Width := 5;
  Letters[69].Data := '00100' +
                      '01110' +
                      '10001' +
                      '11111' +
                      '10001';


  FCaption := 'PIXELFONT BY ESPEEDY.DK';
  FSizeFactor := 1;
end;

function TPixelFont.GetIDFromChar(aChar: Char): Integer;
var
  I: Integer;
begin
  Result := 0;
  aChar :=  AnsiUpperCase(string(aChar))[1];
  for I := 0 to (Length(Letters) - 1) do
    if Letters[I].Letter = aChar then
      begin
        Result := I;
        Exit;
      end;
end;


procedure TPixelFont.Paint;
var
  I, J, K, Bredde: Integer;
  Linier: Array[1..5] of string;
begin
  inherited;
  Bredde := 0;
  for I := 0 to (Length(Caption)-1) do
    begin
      Bredde := Bredde + Letters[GetIDFromChar(Caption[I+1])].Width + 1;
    end;
  Self.Width := SizeFactor * Bredde;
  Self.Height := SizeFactor * 5;
  Bredde := 0;
  for I := 0 to (Length(Caption)-1) do
    begin
      for J := 0 to 4 do
        Linier[J+1] := '';
        
      for J := 0 to 4 do
        for K := 1 to (Letters[GetIDFromChar(Caption[I+1])].Width) do
          if Letters[GetIDFromChar(Caption[I+1])].Data[((J*Letters[GetIDFromChar(Caption[I+1])].Width)+K)] = '1' then
            SetPixel(Bredde + (K-1), J, SizeFactor, Color);

      Bredde := Bredde + Letters[GetIDFromChar(Caption[I+1])].Width + 1;
    end;
end;

procedure TPixelFont.SaveToFile(Filename: TFileName);
var
  Bitmap: TBitmap;
  X, Y: Integer;
begin
  Bitmap := TBitmap.Create;
  Bitmap.Width := Self.Width;
  Bitmap.Height := Self.Height;
  for X := 0 to Self.Width - 1 do
    for Y := 0 to Self.Height - 1 do
      begin
        Bitmap.Canvas.Pixels[X, Y] := Self.Canvas.Pixels[X, Y];
      end;
  Bitmap.SaveToFile(FileName);
end;

procedure TPixelFont.SetCaption(const Value: string);
begin
  FCaption := Value;
  Paint;
end;

procedure TPixelFont.SetColor(const Value: TColor);
begin
  FColor := Value;
  Paint;
end;

procedure TPixelFont.SetPixel(X, Y, aSizeFactor: Integer; Color: TColor);
var
  I, J: Integer;
begin
  for I := 0 to aSizeFactor - 1 do
    for J := 0 to aSizeFactor - 1 do
      begin
        Canvas.Pixels[(X * aSizeFactor) + I, (Y * aSizeFactor) + J] := Color;
      end;
end;

procedure TPixelFont.SetSizeFactor(const Value: Integer);
begin
  if Value > 0 then
    begin
      FSizeFactor := Value;
      Paint;
    end;
end;

end.
